@@ -51,7 +51,7 @@
 #include "SleepCommand.h"
 #include "StreamCheckIntegrityEntry.h"
 #include "PieceStorage.h"
-#include "Socket.h"
+#include "SocketCore.h"
 #include "message.h"
 #include "prefs.h"
 #include "fmt.h"
@@ -69,6 +69,7 @@
 #include "SocketRecvBuffer.h"
 #ifdef ENABLE_ASYNC_DNS
 #include "AsyncNameResolver.h"
+#include "AsyncNameResolverMan.h"
 #endif // ENABLE_ASYNC_DNS
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "ChecksumCheckIntegrityEntry.h"
@@ -82,7 +83,7 @@ AbstractCommand::AbstractCommand
  const SharedHandle<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
  DownloadEngine* e,
- const SocketHandle& s,
+ const SharedHandle<SocketCore>& s,
  const SharedHandle<SocketRecvBuffer>& socketRecvBuffer,
  bool incNumConnection)
   : Command(cuid), checkPoint_(global::wallclock()),
@@ -90,8 +91,10 @@ AbstractCommand::AbstractCommand
     requestGroup_(requestGroup),
     req_(req), fileEntry_(fileEntry), e_(e), socket_(s),
     socketRecvBuffer_(socketRecvBuffer),
+#ifdef ENABLE_ASYNC_DNS
+    asyncNameResolverMan_(new AsyncNameResolverMan()),
+#endif // ENABLE_ASYNC_DNS
     checkSocketIsReadable_(false), checkSocketIsWritable_(false),
-    nameResolverCheck_(false),
     incNumConnection_(incNumConnection),
     serverStatTimer_(global::wallclock())
 {
@@ -103,13 +106,16 @@ AbstractCommand::AbstractCommand
   }
   requestGroup_->increaseStreamCommand();
   requestGroup_->increaseNumCommand();
+#ifdef ENABLE_ASYNC_DNS
+  configureAsyncNameResolverMan(asyncNameResolverMan_.get(), e_->getOption());
+#endif // ENABLE_ASYNC_DNS
 }
 
 AbstractCommand::~AbstractCommand() {
   disableReadCheckSocket();
   disableWriteCheckSocket();
 #ifdef ENABLE_ASYNC_DNS
-  disableNameResolverCheck(asyncNameResolver_);
+  asyncNameResolverMan_->disableNameResolverCheck(e_, this);
 #endif // ENABLE_ASYNC_DNS
   requestGroup_->decreaseNumCommand();
   requestGroup_->decreaseStreamCommand();
@@ -170,9 +176,10 @@ bool AbstractCommand::execute() {
         return prepareForRetry(0);
       }
       // TODO it is not needed to check other PeerStats every time.
-      // Find faster Request when no segment is available.
+      // Find faster Request when no segment split is allowed.
       if(req_ && fileEntry_->countPooledRequest() > 0 &&
-         !getPieceStorage()->hasMissingUnusedPiece()) {
+         requestGroup_->getTotalLength()-requestGroup_->getCompletedLength()
+         < calculateMinSplitSize()*2) {
         SharedHandle<Request> fasterRequest = fileEntry_->findFasterRequest(req_);
         if(fasterRequest) {
           useFasterRequest(fasterRequest);
@@ -205,10 +212,14 @@ bool AbstractCommand::execute() {
        (checkSocketIsWritable_ && writeEventEnabled()) ||
        hupEventEnabled() ||
 #ifdef ENABLE_ASYNC_DNS
-       (nameResolverCheck_ && nameResolveFinished()) ||
+       (asyncNameResolverMan_->resolverChecked() &&
+        asyncNameResolverMan_->getStatus() != 0) ||
 #endif // ENABLE_ASYNC_DNS
-       (!checkSocketIsReadable_ && !checkSocketIsWritable_ &&
-        !nameResolverCheck_)) {
+       (!checkSocketIsReadable_ && !checkSocketIsWritable_
+#ifdef ENABLE_ASYNC_DNS
+        && !asyncNameResolverMan_->resolverChecked()
+#endif // ENABLE_ASYNC_DNS
+        ) || noCheck()) {
       checkPoint_ = global::wallclock();
       if(getPieceStorage()) {
         if(!req_ || req_->getMaxPipelinedRequest() == 1 ||
@@ -431,7 +442,7 @@ void AbstractCommand::onAbort() {
     // limitation of current implementation.
     if(!getOption()->getAsBool(PREF_ALWAYS_RESUME) &&
        fileEntry_ &&
-       getSegmentMan()->calculateSessionDownloadLength() == 0 &&
+       getDownloadContext()->getNetStat().getSessionDownloadLength() == 0 &&
        !requestGroup_->p2pInvolved() &&
        getDownloadContext()->getFileEntries().size() == 1) {
       const int maxTries = getOption()->getAsInt(PREF_MAX_RESUME_FAILURE_TRIES);
@@ -475,10 +486,11 @@ void AbstractCommand::disableReadCheckSocket() {
     e_->deleteSocketForReadCheck(readCheckTarget_, this);
     checkSocketIsReadable_ = false;
     readCheckTarget_.reset();
-  }  
+  }
 }
 
-void AbstractCommand::setReadCheckSocket(const SocketHandle& socket) {
+void AbstractCommand::setReadCheckSocket
+(const SharedHandle<SocketCore>& socket) {
   if(!socket->isOpen()) {
     disableReadCheckSocket();
   } else {
@@ -514,7 +526,8 @@ void AbstractCommand::disableWriteCheckSocket() {
   }
 }
 
-void AbstractCommand::setWriteCheckSocket(const SocketHandle& socket) {
+void AbstractCommand::setWriteCheckSocket
+(const SharedHandle<SocketCore>& socket) {
   if(!socket->isOpen()) {
     disableWriteCheckSocket();
   } else {
@@ -542,6 +555,13 @@ void AbstractCommand::setWriteCheckSocketIf
   }
 }
 
+void AbstractCommand::swapSocket(SharedHandle<SocketCore>& socket)
+{
+  disableReadCheckSocket();
+  disableWriteCheckSocket();
+  socket_.swap(socket);
+}
+
 namespace {
 // Constructs proxy URI, merging username and password if they are
 // defined.
@@ -590,17 +610,17 @@ std::string getProxyOptionFor
 std::string getProxyUri
 (const std::string& protocol, const Option* option)
 {
-  if(protocol == Request::PROTO_HTTP) {
+  if(protocol == "http") {
     return getProxyOptionFor(PREF_HTTP_PROXY,
                              PREF_HTTP_PROXY_USER,
                              PREF_HTTP_PROXY_PASSWD,
                              option);
-  } else if(protocol == Request::PROTO_HTTPS) {
+  } else if(protocol == "https") {
     return getProxyOptionFor(PREF_HTTPS_PROXY,
                              PREF_HTTPS_PROXY_USER,
                              PREF_HTTPS_PROXY_PASSWD,
                              option);
-  } else if(protocol == Request::PROTO_FTP) {
+  } else if(protocol == "ftp") {
     return getProxyOptionFor(PREF_FTP_PROXY,
                              PREF_FTP_PROXY_USER,
                              PREF_FTP_PROXY_PASSWD,
@@ -685,86 +705,6 @@ SharedHandle<Request> AbstractCommand::createProxyRequest() const
   return proxyRequest;
 }
 
-#ifdef ENABLE_ASYNC_DNS
-
-bool AbstractCommand::isAsyncNameResolverInitialized() const
-{
-  return asyncNameResolver_;
-}
-
-void AbstractCommand::initAsyncNameResolver(const std::string& hostname)
-{
-  int family;
-  if(getOption()->getAsBool(PREF_ENABLE_ASYNC_DNS6)) {
-    family = AF_UNSPEC;
-  } else {
-    family = AF_INET;
-  }
-  asyncNameResolver_.reset
-    (new AsyncNameResolver(family
-#ifdef HAVE_ARES_ADDR_NODE
-                           ,
-                           e_->getAsyncDNSServers()
-#endif // HAVE_ARES_ADDR_NODE
-                           ));
-  A2_LOG_INFO(fmt(MSG_RESOLVING_HOSTNAME,
-                  getCuid(),
-                  hostname.c_str()));
-  asyncNameResolver_->resolve(hostname);
-  setNameResolverCheck(asyncNameResolver_);
-}
-
-bool AbstractCommand::asyncResolveHostname()
-{
-  switch(asyncNameResolver_->getStatus()) {
-  case AsyncNameResolver::STATUS_SUCCESS:
-    disableNameResolverCheck(asyncNameResolver_);
-    return true;
-  case AsyncNameResolver::STATUS_ERROR:
-    disableNameResolverCheck(asyncNameResolver_);
-    if(!isProxyRequest(req_->getProtocol(), getOption())) {
-      e_->getRequestGroupMan()->getOrCreateServerStat
-        (req_->getHost(), req_->getProtocol())->setError();
-    }
-    throw DL_ABORT_EX2
-      (fmt(MSG_NAME_RESOLUTION_FAILED,
-           getCuid(),
-           asyncNameResolver_->getHostname().c_str(),
-           asyncNameResolver_->getError().c_str()),
-       error_code::NAME_RESOLVE_ERROR);
-  default:
-    return false;
-  }
-}
-
-const std::vector<std::string>& AbstractCommand::getResolvedAddresses()
-{
-  return asyncNameResolver_->getResolvedAddresses();
-}
-
-void AbstractCommand::setNameResolverCheck
-(const SharedHandle<AsyncNameResolver>& resolver) {
-  if(resolver) {
-    nameResolverCheck_ = true;
-    e_->addNameResolverCheck(resolver, this);
-  }
-}
-
-void AbstractCommand::disableNameResolverCheck
-(const SharedHandle<AsyncNameResolver>& resolver) {
-  if(resolver) {
-    nameResolverCheck_ = false;
-    e_->deleteNameResolverCheck(resolver, this);
-  }
-}
-
-bool AbstractCommand::nameResolveFinished() const {
-  return
-    asyncNameResolver_->getStatus() ==  AsyncNameResolver::STATUS_SUCCESS ||
-    asyncNameResolver_->getStatus() == AsyncNameResolver::STATUS_ERROR;
-}
-#endif // ENABLE_ASYNC_DNS
-
 std::string AbstractCommand::resolveHostname
 (std::vector<std::string>& addrs, const std::string& hostname, uint16_t port)
 {
@@ -777,13 +717,30 @@ std::string AbstractCommand::resolveHostname
   if(addrs.empty()) {
 #ifdef ENABLE_ASYNC_DNS
     if(getOption()->getAsBool(PREF_ASYNC_DNS)) {
-      if(!isAsyncNameResolverInitialized()) {
-        initAsyncNameResolver(hostname);
+      if(!asyncNameResolverMan_->started()) {
+        asyncNameResolverMan_->startAsync(hostname, e_, this);
       }
-      if(asyncResolveHostname()) {
-        addrs = getResolvedAddresses();
-      } else {
+      switch(asyncNameResolverMan_->getStatus()) {
+      case -1:
+        if(!isProxyRequest(req_->getProtocol(), getOption())) {
+          e_->getRequestGroupMan()->getOrCreateServerStat
+            (req_->getHost(), req_->getProtocol())->setError();
+        }
+        throw DL_ABORT_EX2
+          (fmt(MSG_NAME_RESOLUTION_FAILED, getCuid(), hostname.c_str(),
+               asyncNameResolverMan_->getLastError().c_str()),
+           error_code::NAME_RESOLVE_ERROR);
+      case 0:
         return A2STR::NIL;
+      case 1:
+        asyncNameResolverMan_->getResolvedAddress(addrs);
+        if(addrs.empty()) {
+          throw DL_ABORT_EX2
+            (fmt(MSG_NAME_RESOLUTION_FAILED, getCuid(), hostname.c_str(),
+                 "No address returned"),
+             error_code::NAME_RESOLVE_ERROR);
+        }
+        break;
       }
     } else
 #endif // ENABLE_ASYNC_DNS
@@ -831,32 +788,30 @@ bool AbstractCommand::checkIfConnectionEstablished
  const std::string& connectedAddr,
  uint16_t connectedPort)
 {
-  if(socket->isReadable(0)) {
-    std::string error = socket->getSocketError();
-    if(!error.empty()) {
-      // See also InitiateConnectionCommand::executeInternal()
-      e_->markBadIPAddress(connectedHostname, connectedAddr, connectedPort);
-      if(!e_->findCachedIPAddress(connectedHostname, connectedPort).empty()) {
-        A2_LOG_INFO(fmt(MSG_CONNECT_FAILED_AND_RETRY,
-                        getCuid(),
-                        connectedAddr.c_str(), connectedPort));
-        Command* command =
-          InitiateConnectionCommandFactory::createInitiateConnectionCommand
-          (getCuid(), req_, fileEntry_, requestGroup_, e_);
-        e_->setNoWait(true);
-        e_->addCommand(command);
-        return false;
-      }
-      e_->removeCachedIPAddress(connectedHostname, connectedPort);
-      // Don't set error if proxy server is used and its method is GET.
-      if(resolveProxyMethod(req_->getProtocol()) != V_GET ||
-         !isProxyRequest(req_->getProtocol(), getOption())) {
-        e_->getRequestGroupMan()->getOrCreateServerStat
-          (req_->getHost(), req_->getProtocol())->setError();
-      }
-      throw DL_RETRY_EX
-        (fmt(MSG_ESTABLISHING_CONNECTION_FAILED, error.c_str()));
+  std::string error = socket->getSocketError();
+  if(!error.empty()) {
+    // See also InitiateConnectionCommand::executeInternal()
+    e_->markBadIPAddress(connectedHostname, connectedAddr, connectedPort);
+    if(!e_->findCachedIPAddress(connectedHostname, connectedPort).empty()) {
+      A2_LOG_INFO(fmt(MSG_CONNECT_FAILED_AND_RETRY,
+                      getCuid(),
+                      connectedAddr.c_str(), connectedPort));
+      Command* command =
+        InitiateConnectionCommandFactory::createInitiateConnectionCommand
+        (getCuid(), req_, fileEntry_, requestGroup_, e_);
+      e_->setNoWait(true);
+      e_->addCommand(command);
+      return false;
     }
+    e_->removeCachedIPAddress(connectedHostname, connectedPort);
+    // Don't set error if proxy server is used and its method is GET.
+    if(resolveProxyMethod(req_->getProtocol()) != V_GET ||
+       !isProxyRequest(req_->getProtocol(), getOption())) {
+      e_->getRequestGroupMan()->getOrCreateServerStat
+        (req_->getHost(), req_->getProtocol())->setError();
+    }
+    throw DL_RETRY_EX
+      (fmt(MSG_ESTABLISHING_CONNECTION_FAILED, error.c_str()));
   }
   return true;
 }
@@ -865,7 +820,7 @@ const std::string& AbstractCommand::resolveProxyMethod
 (const std::string& protocol) const
 {
   if(getOption()->get(PREF_PROXY_METHOD) == V_TUNNEL ||
-     Request::PROTO_HTTPS == protocol) {
+     protocol == "https") {
     return V_TUNNEL;
   } else {
     return V_GET;
