@@ -41,9 +41,11 @@
 #include "Segment.h"
 #include "Option.h"
 #include "prefs.h"
-#include "Socket.h"
+#include "SocketCore.h"
 #include "DownloadContext.h"
 #include "SocketRecvBuffer.h"
+#include "fmt.h"
+#include "LogFactory.h"
 
 namespace aria2 {
 
@@ -54,7 +56,7 @@ AbstractProxyRequestCommand::AbstractProxyRequestCommand
  RequestGroup* requestGroup,
  DownloadEngine* e,
  const SharedHandle<Request>& proxyRequest,
- const SocketHandle& s)
+ const SharedHandle<SocketCore>& s)
   :
   AbstractCommand(cuid, req, fileEntry, requestGroup, e, s),
   proxyRequest_(proxyRequest),
@@ -72,11 +74,6 @@ AbstractProxyRequestCommand::~AbstractProxyRequestCommand() {}
 bool AbstractProxyRequestCommand::executeInternal() {
   //socket->setBlockingMode();
   if(httpConnection_->sendBufferIsEmpty()) {
-    if(!checkIfConnectionEstablished
-       (getSocket(), getRequest()->getConnectedHostname(),
-        getRequest()->getConnectedAddr(), getRequest()->getConnectedPort())) {
-      return true;
-    }
     SharedHandle<HttpRequest> httpRequest(new HttpRequest());
     httpRequest->setUserAgent(getOption()->get(PREF_USER_AGENT));
     httpRequest->setRequest(getRequest());
