@@ -36,25 +36,29 @@
 #include "Peer.h"
 #include "DHTGetPeersReplyMessage.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "DHTMessageFactory.h"
 #include "DHTNode.h"
 #include "DHTNodeLookupEntry.h"
 #include "DHTMessageDispatcher.h"
 #include "DHTMessageCallback.h"
 #include "PeerStorage.h"
-#include "BtRuntime.h"
 #include "util.h"
 #include "DHTBucket.h"
 #include "bittorrent_helper.h"
 #include "DHTPeerLookupTaskCallback.h"
 #include "DHTQueryMessage.h"
+#include "fmt.h"
 
 namespace aria2 {
 
 DHTPeerLookupTask::DHTPeerLookupTask
-(const SharedHandle<DownloadContext>& downloadContext):
-  DHTAbstractNodeLookupTask<DHTGetPeersReplyMessage>
-  (bittorrent::getInfoHash(downloadContext)) {}
+(const SharedHandle<DownloadContext>& downloadContext,
+ uint16_t tcpPort)
+  : DHTAbstractNodeLookupTask<DHTGetPeersReplyMessage>
+    (bittorrent::getInfoHash(downloadContext)),
+    tcpPort_(tcpPort)
+{}
 
 void
 DHTPeerLookupTask::getNodesFromMessage
@@ -65,7 +69,7 @@ DHTPeerLookupTask::getNodesFromMessage
     message->getClosestKNodes();
   nodes.insert(nodes.end(), knodes.begin(), knodes.end());
 }
-  
+
 void DHTPeerLookupTask::onReceivedInternal
 (const DHTGetPeersReplyMessage* message)
 {
@@ -73,12 +77,10 @@ void DHTPeerLookupTask::onReceivedInternal
   tokenStorage_[util::toHex(remoteNode->getID(), DHT_ID_LENGTH)] =
     message->getToken();
   peerStorage_->addPeer(message->getValues());
-  peers_.insert(peers_.end(),
-                message->getValues().begin(), message->getValues().end());
-  getLogger()->info("Received %lu peers.",
-                    static_cast<unsigned long>(message->getValues().size()));
+  A2_LOG_INFO(fmt("Received %lu peers.",
+                  static_cast<unsigned long>(message->getValues().size())));
 }
-  
+
 SharedHandle<DHTMessage> DHTPeerLookupTask::createMessage
 (const SharedHandle<DHTNode>& remoteNode)
 {
@@ -93,33 +95,34 @@ SharedHandle<DHTMessageCallback> DHTPeerLookupTask::createCallback()
 
 void DHTPeerLookupTask::onFinish()
 {
-  if(getLogger()->debug()) {
-    getLogger()->debug("Peer lookup for %s finished",
-                       util::toHex(getTargetID(), DHT_ID_LENGTH).c_str());
-  }
+  A2_LOG_DEBUG(fmt("Peer lookup for %s finished",
+                   util::toHex(getTargetID(), DHT_ID_LENGTH).c_str()));
   // send announce_peer message to K closest nodes
   size_t num = DHTBucket::K;
   for(std::deque<SharedHandle<DHTNodeLookupEntry> >::const_iterator i =
         getEntries().begin(), eoi = getEntries().end();
-      i != eoi && num > 0; ++i, --num) {
-    if((*i)->used) {
-      const SharedHandle<DHTNode>& node = (*i)->node;
-      SharedHandle<DHTMessage> m = 
-        getMessageFactory()->createAnnouncePeerMessage
-        (node,
-         getTargetID(), // this is infoHash
-         btRuntime_->getListenPort(),
-         tokenStorage_[util::toHex(node->getID(), DHT_ID_LENGTH)]);
-      getMessageDispatcher()->addMessageToQueue(m);
+      i != eoi && num > 0; ++i) {
+    if(!(*i)->used) {
+      continue;
     }
+    const SharedHandle<DHTNode>& node = (*i)->node;
+    std::string idHex = util::toHex(node->getID(), DHT_ID_LENGTH);
+    std::string token = tokenStorage_[idHex];
+    if(token.empty()) {
+      A2_LOG_DEBUG(fmt("Token is empty for ID:%s", idHex.c_str()));
+      continue;
+    }
+    SharedHandle<DHTMessage> m =
+      getMessageFactory()->createAnnouncePeerMessage
+      (node,
+       getTargetID(), // this is infoHash
+       tcpPort_,
+       token);
+    getMessageDispatcher()->addMessageToQueue(m);
+    --num;
   }
 }
 
-void DHTPeerLookupTask::setBtRuntime(const SharedHandle<BtRuntime>& btRuntime)
-{
-  btRuntime_ = btRuntime;
-}
-
 void DHTPeerLookupTask::setPeerStorage(const SharedHandle<PeerStorage>& ps)
 {
   peerStorage_ = ps;
