@@ -52,7 +52,7 @@ public:
 
   virtual SharedHandle<DHTTask>
   createPingTask(const SharedHandle<DHTNode>& remoteNode,
-                 size_t numRetry = 0) = 0;
+                 int numRetry = 0) = 0;
 
   virtual SharedHandle<DHTTask>
   createNodeLookupTask(const unsigned char* targetID) = 0;
@@ -63,7 +63,7 @@ public:
   createPeerLookupTask(const SharedHandle<DownloadContext>& ctx,
                        uint16_t tcpPort,
                        const SharedHandle<PeerStorage>& peerStorage) = 0;
-  
+
   virtual SharedHandle<DHTTask>
   createPeerAnnounceTask(const unsigned char* infoHash) = 0;
 
