@@ -38,10 +38,12 @@
 #include "common.h"
 
 #include <string>
-#include <deque>
+#include <set>
 #include <algorithm>
 #include <vector>
 
+#include "a2functional.h"
+
 namespace aria2 {
 
 class DNSCache {
@@ -68,7 +70,7 @@ private:
 
     CacheEntry& operator=(const CacheEntry& c);
 
-    void add(const std::string& addr);
+    bool add(const std::string& addr);
 
     std::vector<AddrEntry>::iterator find(const std::string& addr);
 
@@ -86,7 +88,7 @@ private:
         if((*i).good_) {
           *out++ = (*i).addr_;
         }
-      }      
+      }
     }
 
     void markBad(const std::string& addr);
@@ -96,8 +98,9 @@ private:
     bool operator==(const CacheEntry& e) const;
   };
 
-  std::deque<CacheEntry> entries_;
-
+  typedef std::set<SharedHandle<CacheEntry>,
+                   DerefLess<SharedHandle<CacheEntry> > > CacheEntrySet;
+  CacheEntrySet entries_;
 public:
   DNSCache();
   DNSCache(const DNSCache& c);
@@ -106,16 +109,15 @@ public:
   DNSCache& operator=(const DNSCache& c);
 
   const std::string& find(const std::string& hostname, uint16_t port) const;
-  
+
   template<typename OutputIterator>
   void findAll
   (OutputIterator out, const std::string& hostname, uint16_t port) const
   {
-    CacheEntry target(hostname, port);
-    std::deque<CacheEntry>::const_iterator i =
-      std::lower_bound(entries_.begin(), entries_.end(), target);
-    if(i != entries_.end() && (*i) == target) {
-      (*i).getAllGoodAddrs(out);
+    SharedHandle<CacheEntry> target(new CacheEntry(hostname, port));
+    CacheEntrySet::iterator i = entries_.find(target);
+    if(i != entries_.end()) {
+      (*i)->getAllGoodAddrs(out);
     }
   }
 
