@@ -41,7 +41,6 @@ namespace aria2 {
 
 class PeerStorage;
 class Peer;
-class Logger;
 class ExtensionMessageRegistry;
 class DownloadContext;
 class BtMessageFactory;
@@ -58,14 +57,11 @@ private:
 
   SharedHandle<DownloadContext> dctx_;
 
-  WeakHandle<BtMessageFactory> messageFactory_;
+  BtMessageFactory* messageFactory_;
 
-  WeakHandle<BtMessageDispatcher> dispatcher_;
-
-  WeakHandle<UTMetadataRequestTracker> tracker_;
-
-  Logger* logger_;
+  BtMessageDispatcher* dispatcher_;
 
+  UTMetadataRequestTracker* tracker_;
 public:
   DefaultExtensionMessageFactory();
 
@@ -93,25 +89,22 @@ public:
     dctx_ = dctx;
   }
 
-  void setBtMessageFactory(const WeakHandle<BtMessageFactory>& factory)
+  void setBtMessageFactory(BtMessageFactory* factory)
   {
     messageFactory_ = factory;
   }
 
-  void setBtMessageDispatcher(const WeakHandle<BtMessageDispatcher>& disp)
+  void setBtMessageDispatcher(BtMessageDispatcher* disp)
   {
     dispatcher_ = disp;
   }
-  
-  void setUTMetadataRequestTracker
-  (const WeakHandle<UTMetadataRequestTracker>& tracker)
+
+  void setUTMetadataRequestTracker(UTMetadataRequestTracker* tracker)
   {
     tracker_ = tracker;
   }
 };
 
-typedef SharedHandle<DefaultExtensionMessageFactory> DefaultExtensionMessageFactoryHandle;
-
 } // namespace aria2
 
 #endif // D_DEFAULT_EXTENSION_MESSAGE_FACTORY_H
