@@ -36,8 +36,6 @@
 
 #include <algorithm>
 
-#include "LogFactory.h"
-#include "Logger.h"
 #include "Option.h"
 #include "RequestGroup.h"
 #include "DownloadEngine.h"
@@ -52,13 +50,18 @@
 #include "FillRequestGroupCommand.h"
 #include "FileAllocationDispatcherCommand.h"
 #include "AutoSaveCommand.h"
+#include "SaveSessionCommand.h"
 #include "HaveEraseCommand.h"
 #include "TimedHaltCommand.h"
+#include "WatchProcessCommand.h"
 #include "DownloadResult.h"
 #include "ServerStatMan.h"
 #include "a2io.h"
 #include "DownloadContext.h"
 #include "array_fun.h"
+#ifdef HAVE_LIBUV
+# include "LibuvEventPoll.h"
+#endif // HAVE_LIBUV
 #ifdef HAVE_EPOLL
 # include "EpollEventPoll.h"
 #endif // HAVE_EPOLL
@@ -74,16 +77,14 @@
 #include "SelectEventPoll.h"
 #include "DlAbortEx.h"
 #include "FileAllocationEntry.h"
-#ifdef ENABLE_XML_RPC
-# include "HttpListenCommand.h"
-#endif // ENABLE_XML_RPC
+#include "HttpListenCommand.h"
+#include "LogFactory.h"
 
 namespace aria2 {
 
-DownloadEngineFactory::DownloadEngineFactory():
-  logger_(LogFactory::getInstance()) {}
+DownloadEngineFactory::DownloadEngineFactory() {}
 
-DownloadEngineHandle
+SharedHandle<DownloadEngine>
 DownloadEngineFactory::newDownloadEngine
 (Option* op, const std::vector<SharedHandle<RequestGroup> >& requestGroups)
 {
@@ -91,6 +92,17 @@ DownloadEngineFactory::newDownloadEngine
     op->getAsInt(PREF_MAX_CONCURRENT_DOWNLOADS);
   SharedHandle<EventPoll> eventPoll;
   const std::string& pollMethod = op->get(PREF_EVENT_POLL);
+#ifdef HAVE_LIBUV
+  if (pollMethod == V_LIBUV) {
+    SharedHandle<LibuvEventPoll> ep(new LibuvEventPoll());
+    if (!ep->good()) {
+      throw DL_ABORT_EX("Initializing LibuvEventPoll failed."
+                        " Try --event-poll=select");
+    }
+    eventPoll = ep;
+  }
+  else
+#endif // HAVE_LIBUV
 #ifdef HAVE_EPOLL
   if(pollMethod == V_EPOLL) {
     SharedHandle<EpollEventPoll> ep(new EpollEventPoll());
@@ -134,12 +146,13 @@ DownloadEngineFactory::newDownloadEngine
           } else {
             abort();
           }
-  DownloadEngineHandle e(new DownloadEngine(eventPoll));
+  SharedHandle<DownloadEngine> e(new DownloadEngine(eventPoll));
   e->setOption(op);
 
-  RequestGroupManHandle
+  SharedHandle<RequestGroupMan>
     requestGroupMan(new RequestGroupMan(requestGroups, MAX_CONCURRENT_DOWNLOADS,
                                         op));
+  requestGroupMan->initWrDiskCache();
   e->setRequestGroupMan(requestGroupMan);
   e->setFileAllocationMan
     (SharedHandle<FileAllocationMan>(new FileAllocationMan()));
@@ -160,6 +173,11 @@ DownloadEngineFactory::newDownloadEngine
       (new AutoSaveCommand(e->newCUID(), e.get(),
                            op->getAsInt(PREF_AUTO_SAVE_INTERVAL)));
   }
+  if(op->getAsInt(PREF_SAVE_SESSION_INTERVAL) > 0) {
+    e->addRoutineCommand
+      (new SaveSessionCommand(e->newCUID(), e.get(),
+                              op->getAsInt(PREF_SAVE_SESSION_INTERVAL)));
+  }
   e->addRoutineCommand(new HaveEraseCommand(e->newCUID(), e.get(), 10));
   {
     time_t stopSec = op->getAsInt(PREF_STOP);
@@ -168,21 +186,32 @@ DownloadEngineFactory::newDownloadEngine
                                                 stopSec));
     }
   }
-#ifdef ENABLE_XML_RPC
-  if(op->getAsBool(PREF_ENABLE_XML_RPC)) {
+  if(op->defined(PREF_STOP_WITH_PROCESS)) {
+    unsigned int pid = op->getAsInt(PREF_STOP_WITH_PROCESS);
+    e->addRoutineCommand(new WatchProcessCommand(e->newCUID(), e.get(), pid));
+  }
+  if(op->getAsBool(PREF_ENABLE_RPC)) {
+    bool ok = false;
+    bool secure = op->getAsBool(PREF_RPC_SECURE);
+    if(secure) {
+      A2_LOG_NOTICE("RPC transport will be encrypted.");
+    }
     static int families[] = { AF_INET, AF_INET6 };
     size_t familiesLength = op->getAsBool(PREF_DISABLE_IPV6)?1:2;
     for(size_t i = 0; i < familiesLength; ++i) {
       HttpListenCommand* httpListenCommand =
-        new HttpListenCommand(e->newCUID(), e.get(), families[i]);
-      if(httpListenCommand->bindPort(op->getAsInt(PREF_XML_RPC_LISTEN_PORT))){
-        e->addRoutineCommand(httpListenCommand);
+        new HttpListenCommand(e->newCUID(), e.get(), families[i], secure);
+      if(httpListenCommand->bindPort(op->getAsInt(PREF_RPC_LISTEN_PORT))){
+        e->addCommand(httpListenCommand);
+        ok = true;
       } else {
         delete httpListenCommand;
       }
     }
+    if(!ok) {
+      throw DL_ABORT_EX("Failed to setup RPC server.");
+    }
   }
-#endif // ENABLE_XML_RPC
   return e;
 }
 
