@@ -50,27 +50,30 @@
 #include "AuthConfig.h"
 #include "DlRetryEx.h"
 #include "DlAbortEx.h"
-#include "Socket.h"
+#include "SocketCore.h"
 #include "A2STR.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "AuthConfig.h"
 #include "a2functional.h"
+#include "util.h"
+#include "error_code.h"
 
 namespace aria2 {
 
-const std::string FtpConnection::A("A");
-
-const std::string FtpConnection::I("I");
-
-FtpConnection::FtpConnection(cuid_t cuid, const SocketHandle& socket,
-                             const SharedHandle<Request>& req,
-                             const SharedHandle<AuthConfig>& authConfig,
-                             const Option* op):
-  cuid_(cuid), socket_(socket), req_(req),
-  authConfig_(authConfig), option_(op),
-  logger_(LogFactory::getInstance()),
-  socketBuffer_(socket),
-  baseWorkingDir_("/") {}
+FtpConnection::FtpConnection
+(cuid_t cuid,
+ const SharedHandle<SocketCore>& socket,
+ const SharedHandle<Request>& req,
+ const SharedHandle<AuthConfig>& authConfig,
+ const Option* op)
+  : cuid_(cuid),
+    socket_(socket),
+    req_(req),
+    authConfig_(authConfig),
+    option_(op),
+    socketBuffer_(socket),
+    baseWorkingDir_("/")
+{}
 
 FtpConnection::~FtpConnection() {}
 
@@ -80,10 +83,8 @@ bool FtpConnection::sendUser()
     std::string request = "USER ";
     request += authConfig_->getUser();
     request += "\r\n";
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(),"USER ********");
-    }
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_, "USER ********"));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -96,10 +97,8 @@ bool FtpConnection::sendPass()
     std::string request = "PASS ";
     request += authConfig_->getPassword();
     request += "\r\n";
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(),"PASS ********");
-    }
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_, "PASS ********"));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -109,19 +108,11 @@ bool FtpConnection::sendPass()
 bool FtpConnection::sendType()
 {
   if(socketBuffer_.sendBufferIsEmpty()) {
-    std::string type;
-    if(option_->get(PREF_FTP_TYPE) == V_ASCII) {
-      type = FtpConnection::A;
-    } else {
-      type = FtpConnection::I;
-    }
     std::string request = "TYPE ";
-    request += type;
+    request += (option_->get(PREF_FTP_TYPE) == V_ASCII ? 'A' : 'I');
     request += "\r\n";
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(),request.c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_,request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -132,10 +123,8 @@ bool FtpConnection::sendPwd()
 {
   if(socketBuffer_.sendBufferIsEmpty()) {
     std::string request = "PWD\r\n";
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(),request.c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_,request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -146,12 +135,10 @@ bool FtpConnection::sendCwd(const std::string& dir)
 {
   if(socketBuffer_.sendBufferIsEmpty()) {
     std::string request = "CWD ";
-    request += util::percentDecode(dir);
+    request += util::percentDecode(dir.begin(), dir.end());
     request += "\r\n";
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(),request.c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_,request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -162,12 +149,11 @@ bool FtpConnection::sendMdtm()
 {
   if(socketBuffer_.sendBufferIsEmpty()) {
     std::string request = "MDTM ";
-    request += util::percentDecode(req_->getFile());
+    request +=
+      util::percentDecode(req_->getFile().begin(), req_->getFile().end());
     request += "\r\n";
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(), request.c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_, request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -178,12 +164,11 @@ bool FtpConnection::sendSize()
 {
   if(socketBuffer_.sendBufferIsEmpty()) {
     std::string request = "SIZE ";
-    request += util::percentDecode(req_->getFile());
+    request +=
+      util::percentDecode(req_->getFile().begin(), req_->getFile().end());
     request += "\r\n";
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(), request.c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_, request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -193,11 +178,9 @@ bool FtpConnection::sendSize()
 bool FtpConnection::sendEpsv()
 {
   if(socketBuffer_.sendBufferIsEmpty()) {
-    static const std::string request("EPSV\r\n");
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(), request.c_str());
-    }
+    std::string request("EPSV\r\n");
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_, request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -207,11 +190,9 @@ bool FtpConnection::sendEpsv()
 bool FtpConnection::sendPasv()
 {
   if(socketBuffer_.sendBufferIsEmpty()) {
-    static const std::string request("PASV\r\n");
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(), request.c_str());
-    }
+    std::string request("PASV\r\n");
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_, request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -223,32 +204,25 @@ SharedHandle<SocketCore> FtpConnection::createServerSocket()
   std::pair<std::string, uint16_t> addrinfo;
   socket_->getAddrInfo(addrinfo);
   SharedHandle<SocketCore> serverSocket(new SocketCore());
-  serverSocket->bind(addrinfo.first, 0, AF_UNSPEC);
+  serverSocket->bind(addrinfo.first.c_str(), 0, AF_UNSPEC);
   serverSocket->beginListen();
-  serverSocket->setNonBlockingMode();
   return serverSocket;
 }
 
 bool FtpConnection::sendEprt(const SharedHandle<SocketCore>& serverSocket)
 {
   if(socketBuffer_.sendBufferIsEmpty()) {
-    struct sockaddr_storage sockaddr;
+    sockaddr_union sockaddr;
     socklen_t len = sizeof(sockaddr);
     serverSocket->getAddrInfo(sockaddr, len);
-    std::pair<std::string, uint16_t> addrinfo = util::getNumericNameInfo
-      (reinterpret_cast<const struct sockaddr*>(&sockaddr), len);
-    std::string request = "EPRT ";
-    request += "|";
-    request += util::itos(sockaddr.ss_family == AF_INET?1:2);
-    request += "|";
-    request += addrinfo.first;
-    request += "|";
-    request += util::uitos(addrinfo.second);
-    request += "|\r\n";
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(), request.c_str());
-    }
+    std::pair<std::string, uint16_t> addrinfo =
+      util::getNumericNameInfo(&sockaddr.sa, len);
+    std::string request =
+      fmt("EPRT |%d|%s|%u|\r\n",
+          sockaddr.storage.ss_family == AF_INET ? 1 : 2,
+          addrinfo.first.c_str(),
+          addrinfo.second);
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST, cuid_, request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -260,27 +234,15 @@ bool FtpConnection::sendPort(const SharedHandle<SocketCore>& serverSocket)
   if(socketBuffer_.sendBufferIsEmpty()) {
     std::pair<std::string, uint16_t> addrinfo;
     socket_->getAddrInfo(addrinfo);
-    unsigned int ipaddr[4]; 
-    sscanf(addrinfo.first.c_str(), "%u.%u.%u.%u",
+    int ipaddr[4];
+    sscanf(addrinfo.first.c_str(), "%d.%d.%d.%d",
            &ipaddr[0], &ipaddr[1], &ipaddr[2], &ipaddr[3]);
     serverSocket->getAddrInfo(addrinfo);
-    std::string request = "PORT ";
-    request += util::uitos(ipaddr[0]);
-    request += ",";
-    request += util::uitos(ipaddr[1]);
-    request += ",";
-    request += util::uitos(ipaddr[2]);
-    request += ",";
-    request += util::uitos(ipaddr[3]);
-    request += ",";
-    request += util::uitos(addrinfo.second/256);
-    request += ",";
-    request += util::uitos(addrinfo.second%256);
-    request += "\r\n";
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(), request.c_str());
-    }
+    std::string request = fmt("PORT %d,%d,%d,%d,%d,%d\r\n",
+                              ipaddr[0], ipaddr[1], ipaddr[2], ipaddr[3],
+                              addrinfo.second/256, addrinfo.second%256);
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_, request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -290,17 +252,12 @@ bool FtpConnection::sendPort(const SharedHandle<SocketCore>& serverSocket)
 bool FtpConnection::sendRest(const SharedHandle<Segment>& segment)
 {
   if(socketBuffer_.sendBufferIsEmpty()) {
-    std::string request = "REST ";
-    if(segment.isNull()) {
-      request += "0";
-    } else {
-      request += util::itos(segment->getPositionToWrite());
-    }
-    request += "\r\n";    
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(), request.c_str());
-    }
+    std::string request =
+      fmt("REST %" PRId64 "\r\n",
+          segment ?
+          segment->getPositionToWrite() : static_cast<int64_t>(0LL));
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_, request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
@@ -311,28 +268,27 @@ bool FtpConnection::sendRetr()
 {
   if(socketBuffer_.sendBufferIsEmpty()) {
     std::string request = "RETR ";
-    request += util::percentDecode(req_->getFile());
+    request +=
+      util::percentDecode(req_->getFile().begin(), req_->getFile().end());
     request += "\r\n";
-    if(logger_->info()) {
-      logger_->info(MSG_SENDING_REQUEST,
-                    util::itos(cuid_).c_str(), request.c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                    cuid_, request.c_str()));
     socketBuffer_.pushStr(request);
   }
   socketBuffer_.send();
   return socketBuffer_.sendBufferIsEmpty();
 }
 
-unsigned int FtpConnection::getStatus(const std::string& response) const
+int FtpConnection::getStatus(const std::string& response) const
 {
-  unsigned int status;
-  // When the response is not like "%u %*s",
+  int status;
+  // When the response is not like "%d %*s",
   // we return 0.
   if(response.find_first_not_of("0123456789") != 3
      || !(response.find(" ") == 3 || response.find("-") == 3)) {
     return 0;
   }
-  if(sscanf(response.c_str(), "%u %*s", &status) == 1) {
+  if(sscanf(response.c_str(), "%d %*s", &status) == 1) {
     return status;
   } else {
     return 0;
@@ -343,8 +299,8 @@ unsigned int FtpConnection::getStatus(const std::string& response) const
 // The length includes \r\n.
 // If the whole response has not been received, then returns std::string::npos.
 std::string::size_type
-FtpConnection::findEndOfResponse(unsigned int status,
-                                 const std::string& buf) const
+FtpConnection::findEndOfResponse
+(int status, const std::string& buf) const
 {
   if(buf.size() <= 4) {
     return std::string::npos;
@@ -353,15 +309,11 @@ FtpConnection::findEndOfResponse(unsigned int status,
   if(buf.at(3) == '-') {
     // multi line response
     std::string::size_type p;
-
-    std::string endPattern = A2STR::CRLF;
-    endPattern += util::uitos(status);
-    endPattern += " ";
-    p = buf.find(endPattern);
+    p = buf.find(fmt("\r\n%d ", status));
     if(p == std::string::npos) {
       return std::string::npos;
     }
-    p = buf.find(A2STR::CRLF, p+6);
+    p = buf.find("\r\n", p+6);
     if(p == std::string::npos) {
       return std::string::npos;
     } else {
@@ -369,7 +321,7 @@ FtpConnection::findEndOfResponse(unsigned int status,
     }
   } else {
     // single line response
-    std::string::size_type p = buf.find(A2STR::CRLF);    
+    std::string::size_type p = buf.find("\r\n");
     if(p == std::string::npos) {
       return std::string::npos;
     } else {
@@ -378,31 +330,32 @@ FtpConnection::findEndOfResponse(unsigned int status,
   }
 }
 
-bool FtpConnection::bulkReceiveResponse
-(std::pair<unsigned int, std::string>& response)
+bool FtpConnection::bulkReceiveResponse(std::pair<int, std::string>& response)
 {
-  char buf[1024];  
-  while(socket_->isReadable(0)) {
+  char buf[1024];
+  while(1) {
     size_t size = sizeof(buf);
     socket_->readData(buf, size);
     if(size == 0) {
       if(socket_->wantRead() || socket_->wantWrite()) {
-        return false;
+        break;
+      } else {
+        throw DL_RETRY_EX(EX_GOT_EOF);
       }
-      throw DL_RETRY_EX(EX_GOT_EOF);
     }
     if(strbuf_.size()+size > MAX_RECV_BUFFER) {
       throw DL_RETRY_EX
-        (StringFormat("Max FTP recv buffer reached. length=%lu",
-                      static_cast<unsigned long>(strbuf_.size()+size)).str());
+        (fmt("Max FTP recv buffer reached. length=%lu",
+             static_cast<unsigned long>(strbuf_.size()+size)));
     }
     strbuf_.append(&buf[0], &buf[size]);
   }
-  unsigned int status;
+  int status;
   if(strbuf_.size() >= 4) {
     status = getStatus(strbuf_);
     if(status == 0) {
-      throw DL_ABORT_EX(EX_INVALID_RESPONSE);
+      throw DL_ABORT_EX2(EX_INVALID_RESPONSE,
+                         error_code::FTP_PROTOCOL_ERROR);
     }
   } else {
     return false;
@@ -410,11 +363,10 @@ bool FtpConnection::bulkReceiveResponse
   std::string::size_type length;
   if((length = findEndOfResponse(status, strbuf_)) != std::string::npos) {
     response.first = status;
-    response.second = strbuf_.substr(0, length);
-    if(logger_->info()) {
-      logger_->info(MSG_RECEIVE_RESPONSE,
-                    util::itos(cuid_).c_str(), response.second.c_str());
-    }
+    response.second.assign(strbuf_.begin(), strbuf_.begin()+length);
+    A2_LOG_INFO(fmt(MSG_RECEIVE_RESPONSE,
+                    cuid_,
+                    response.second.c_str()));
     strbuf_.erase(0, length);
     return true;
   } else {
@@ -423,9 +375,9 @@ bool FtpConnection::bulkReceiveResponse
   }
 }
 
-unsigned int FtpConnection::receiveResponse()
+int FtpConnection::receiveResponse()
 {
-  std::pair<unsigned int, std::string> response;
+  std::pair<int, std::string> response;
   if(bulkReceiveResponse(response)) {
     return response.first;
   } else {
@@ -439,20 +391,25 @@ unsigned int FtpConnection::receiveResponse()
 # define LONGLONG_SCANF "%I64d"
 # define ULONGLONG_SCANF "%I64u"
 #else
-# define LONGLONG_PRINTF "%lld"
+# define LONGLONG_PRINTF "%" PRId64 ""
 # define ULONGLONG_PRINTF "%llu"
 # define LONGLONG_SCANF "%Ld"
 // Mac OSX uses "%llu" for 64bits integer.
 # define ULONGLONG_SCANF "%Lu"
 #endif // __MINGW32__
 
-unsigned int FtpConnection::receiveSizeResponse(uint64_t& size)
+int FtpConnection::receiveSizeResponse(int64_t& size)
 {
-  std::pair<unsigned int, std::string> response;
+  std::pair<int, std::string> response;
   if(bulkReceiveResponse(response)) {
     if(response.first == 213) {
-      std::pair<std::string, std::string> rp = util::split(response.second," ");
-      size = util::parseULLInt(rp.second);
+      std::pair<Sip, Sip> rp;
+      util::divide(rp, response.second.begin(), response.second.end(), ' ');
+      if(!util::parseLLIntNoThrow(size, std::string(rp.second.first,
+                                                    rp.second.second)) ||
+         size < 0) {
+        throw DL_ABORT_EX("Size must be positive integer");
+      }
     }
     return response.first;
   } else {
@@ -460,10 +417,29 @@ unsigned int FtpConnection::receiveSizeResponse(uint64_t& size)
   }
 }
 
-unsigned int FtpConnection::receiveMdtmResponse(Time& time)
+namespace {
+template<typename T>
+bool getInteger(T* dest, const char* first, const char* last)
+{
+  int res = 0;
+  // We assume *dest won't overflow.
+  for(; first != last; ++first) {
+    if(util::isDigit(*first)) {
+      res *= 10;
+      res += (*first)-'0';
+    } else {
+      return false;
+    }
+  }
+  *dest = res;
+  return true;
+}
+} // namespace
+
+int FtpConnection::receiveMdtmResponse(Time& time)
 {
   // MDTM command, specified in RFC3659.
-  std::pair<unsigned int, std::string> response;
+  std::pair<int, std::string> response;
   if(bulkReceiveResponse(response)) {
     if(response.first == 213) {
       char buf[15]; // YYYYMMDDhhmmss+\0, milli second part is dropped.
@@ -473,18 +449,18 @@ unsigned int FtpConnection::receiveMdtmResponse(Time& time)
         // and included strptime doesn't parse data for this format.
         struct tm tm;
         memset(&tm, 0, sizeof(tm));
-        tm.tm_sec = util::parseInt(&buf[12]);
-        buf[12] = '\0';
-        tm.tm_min = util::parseInt(&buf[10]);
-        buf[10] = '\0';
-        tm.tm_hour = util::parseInt(&buf[8]);
-        buf[8] = '\0';
-        tm.tm_mday = util::parseInt(&buf[6]);
-        buf[6] = '\0';
-        tm.tm_mon = util::parseInt(&buf[4])-1;
-        buf[4] = '\0';
-        tm.tm_year = util::parseInt(&buf[0])-1900;
-        time = Time(timegm(&tm));
+        if(getInteger(&tm.tm_sec, &buf[12], &buf[14]) &&
+           getInteger(&tm.tm_min, &buf[10], &buf[12]) &&
+           getInteger(&tm.tm_hour, &buf[8], &buf[10]) &&
+           getInteger(&tm.tm_mday, &buf[6], &buf[8]) &&
+           getInteger(&tm.tm_mon, &buf[4], &buf[6]) &&
+           getInteger(&tm.tm_year, &buf[0], &buf[4])) {
+          tm.tm_mon -= 1;
+          tm.tm_year -= 1900;
+          time = Time(timegm(&tm));
+        } else {
+          time = Time::null();
+        }
       } else {
         time = Time::null();
       }
@@ -495,9 +471,9 @@ unsigned int FtpConnection::receiveMdtmResponse(Time& time)
   }
 }
 
-unsigned int FtpConnection::receiveEpsvResponse(uint16_t& port)
+int FtpConnection::receiveEpsvResponse(uint16_t& port)
 {
-  std::pair<unsigned int, std::string> response;
+  std::pair<int, std::string> response;
   if(bulkReceiveResponse(response)) {
     if(response.first == 229) {
       port = 0;
@@ -507,11 +483,14 @@ unsigned int FtpConnection::receiveEpsvResponse(uint16_t& port)
          leftParen > rightParen) {
         return response.first;
       }
-      std::vector<std::string> rd;
-      util::split(response.second.substr(leftParen+1, rightParen-(leftParen+1)),
-                  std::back_inserter(rd), "|", true, true);
+      std::vector<Scip> rd;
+      util::splitIter(response.second.begin()+leftParen+1,
+                      response.second.begin()+rightParen,
+                      std::back_inserter(rd), '|', true, true);
       uint32_t portTemp = 0;
-      if(rd.size() == 5 && util::parseUIntNoThrow(portTemp, rd[3])) {
+      if(rd.size() == 5 &&
+         util::parseUIntNoThrow(portTemp,
+                                std::string(rd[3].first, rd[3].second))) {
         if(0 < portTemp  && portTemp <= UINT16_MAX) {
           port = portTemp;
         }
@@ -523,28 +502,22 @@ unsigned int FtpConnection::receiveEpsvResponse(uint16_t& port)
   }
 }
 
-unsigned int FtpConnection::receivePasvResponse
+int FtpConnection::receivePasvResponse
 (std::pair<std::string, uint16_t>& dest)
 {
-  std::pair<unsigned int, std::string> response;
+  std::pair<int, std::string> response;
   if(bulkReceiveResponse(response)) {
     if(response.first == 227) {
       // we assume the format of response is "227 Entering Passive
       // Mode (h1,h2,h3,h4,p1,p2)."
-      unsigned int h1, h2, h3, h4, p1, p2;
+      int h1, h2, h3, h4, p1, p2;
       std::string::size_type p = response.second.find("(");
       if(p >= 4) {
-        sscanf(response.second.substr(response.second.find("(")).c_str(),
-               "(%u,%u,%u,%u,%u,%u).",
+        sscanf(response.second.c_str()+p,
+               "(%d,%d,%d,%d,%d,%d).",
                &h1, &h2, &h3, &h4, &p1, &p2);
         // ip address
-        dest.first = util::uitos(h1);
-        dest.first += A2STR::DOT_C;
-        dest.first += util::uitos(h2);
-        dest.first += A2STR::DOT_C;
-        dest.first += util::uitos(h3);
-        dest.first += A2STR::DOT_C;
-        dest.first += util::uitos(h4);
+        dest.first = fmt("%d.%d.%d.%d", h1, h2, h3, h4);
         // port number
         dest.second = 256*p1+p2;
       } else {
@@ -557,9 +530,9 @@ unsigned int FtpConnection::receivePasvResponse
   }
 }
 
-unsigned int FtpConnection::receivePwdResponse(std::string& pwd)
+int FtpConnection::receivePwdResponse(std::string& pwd)
 {
-  std::pair<unsigned int, std::string> response;
+  std::pair<int, std::string> response;
   if(bulkReceiveResponse(response)) {
     if(response.first == 257) {
       std::string::size_type first;
@@ -567,9 +540,10 @@ unsigned int FtpConnection::receivePwdResponse(std::string& pwd)
 
       if((first = response.second.find("\"")) != std::string::npos &&
          (last = response.second.find("\"", ++first)) != std::string::npos) {
-        pwd = response.second.substr(first, last-first);
+        pwd.assign(response.second.begin()+first, response.second.begin()+last);
       } else {
-        throw DL_ABORT_EX(EX_INVALID_RESPONSE);
+        throw DL_ABORT_EX2(EX_INVALID_RESPONSE,
+                           error_code::FTP_PROTOCOL_ERROR);
       }
     }
     return response.first;
