@@ -39,20 +39,17 @@
 #include "DlAbortEx.h"
 #include "message.h"
 #include "prefs.h"
-#include "Socket.h"
+#include "SocketCore.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "Peer.h"
 #include "BtRuntime.h"
 #include "PeerStorage.h"
 #include "PieceStorage.h"
 #include "PeerConnection.h"
 #include "RequestGroup.h"
-#include "DownloadContext.h"
 #include "util.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
-#include "RequestGroupMan.h"
-#include "ServerStatMan.h"
+#include "fmt.h"
 
 namespace aria2 {
 
@@ -63,11 +60,10 @@ PeerInitiateConnectionCommand::PeerInitiateConnectionCommand
  DownloadEngine* e,
  const SharedHandle<BtRuntime>& btRuntime,
  bool mseHandshakeEnabled)
-  :
-  PeerAbstractCommand(cuid, peer, e),
-  requestGroup_(requestGroup),
-  btRuntime_(btRuntime),
-  mseHandshakeEnabled_(mseHandshakeEnabled)
+  : PeerAbstractCommand(cuid, peer, e),
+    requestGroup_(requestGroup),
+    btRuntime_(btRuntime),
+    mseHandshakeEnabled_(mseHandshakeEnabled)
 {
   btRuntime_->increaseConnections();
   requestGroup_->increaseNumCommand();
@@ -80,15 +76,13 @@ PeerInitiateConnectionCommand::~PeerInitiateConnectionCommand()
 }
 
 bool PeerInitiateConnectionCommand::executeInternal() {
-  if(getLogger()->info()) {
-    getLogger()->info(MSG_CONNECTING_TO_SERVER,
-                      util::itos(getCuid()).c_str(),
-                      getPeer()->getIPAddress().c_str(),
-                      getPeer()->getPort());
-  }
+  A2_LOG_INFO(fmt(MSG_CONNECTING_TO_SERVER,
+                  getCuid(),
+                  getPeer()->getIPAddress().c_str(),
+                  getPeer()->getPort()));
   createSocket();
   getSocket()->establishConnection(getPeer()->getIPAddress(),
-                                   getPeer()->getPort());
+                                   getPeer()->getPort(), false);
   if(mseHandshakeEnabled_) {
     InitiatorMSEHandshakeCommand* c =
       new InitiatorMSEHandshakeCommand(getCuid(), requestGroup_, getPeer(),
@@ -111,14 +105,18 @@ bool PeerInitiateConnectionCommand::executeInternal() {
 // TODO this method removed when PeerBalancerCommand is implemented
 bool PeerInitiateConnectionCommand::prepareForNextPeer(time_t wait) {
   if(peerStorage_->isPeerAvailable() && btRuntime_->lessThanEqMinPeers()) {
-    SharedHandle<Peer> peer = peerStorage_->getUnusedPeer();
-    peer->usedBy(getDownloadEngine()->newCUID());
-    PeerInitiateConnectionCommand* command =
-      new PeerInitiateConnectionCommand(peer->usedBy(), requestGroup_, peer,
-                                        getDownloadEngine(), btRuntime_);
-    command->setPeerStorage(peerStorage_);
-    command->setPieceStorage(pieceStorage_);
-    getDownloadEngine()->addCommand(command);
+    cuid_t ncuid = getDownloadEngine()->newCUID();
+    SharedHandle<Peer> peer = peerStorage_->checkoutPeer(ncuid);
+    // sanity check
+    if(peer) {
+      PeerInitiateConnectionCommand* command;
+      command = new PeerInitiateConnectionCommand(ncuid, requestGroup_, peer,
+                                                  getDownloadEngine(),
+                                                  btRuntime_);
+      command->setPeerStorage(peerStorage_);
+      command->setPieceStorage(pieceStorage_);
+      getDownloadEngine()->addCommand(command);
+    }
   }
   return true;
 }
