@@ -45,7 +45,7 @@
 #include "ReceiverMSEHandshakeCommand.h"
 #include "Logger.h"
 #include "LogFactory.h"
-#include "Socket.h"
+#include "SocketCore.h"
 #include "SimpleRandomizer.h"
 #include "util.h"
 #include "fmt.h"
@@ -63,33 +63,28 @@ PeerListenCommand::PeerListenCommand
 
 PeerListenCommand::~PeerListenCommand() {}
 
-bool PeerListenCommand::bindPort(uint16_t& port, IntSequence& seq)
+bool PeerListenCommand::bindPort(uint16_t& port, SegList<int>& sgl)
 {
   socket_.reset(new SocketCore());
-
-  std::vector<int32_t> randPorts = seq.flush();
-  std::random_shuffle(randPorts.begin(), randPorts.end(),
+  std::vector<uint16_t> ports;
+  while(sgl.hasNext()) {
+    ports.push_back(sgl.next());
+  }
+  std::random_shuffle(ports.begin(), ports.end(),
                       *SimpleRandomizer::getInstance().get());
-  
-  for(std::vector<int32_t>::const_iterator portItr = randPorts.begin(),
-        eoi = randPorts.end(); portItr != eoi; ++portItr) {
-    if(!(0 < (*portItr) && (*portItr) <= 65535)) {
-      continue;
-    }
-    port = (*portItr);
+  const int ipv = (family_ == AF_INET) ? 4 : 6;
+  for(std::vector<uint16_t>::const_iterator i = ports.begin(),
+        eoi = ports.end(); i != eoi; ++i) {
+    port = *i;
     try {
-      socket_->bind(A2STR::NIL, port, family_);
+      socket_->bind(0, port, family_);
       socket_->beginListen();
-      socket_->setNonBlockingMode();
-      A2_LOG_NOTICE(fmt("IPv%d BitTorrent: listening to port %d",
-                        family_ == AF_INET?4:6, port));
+      A2_LOG_NOTICE(fmt(_("IPv%d BitTorrent: listening on TCP port %u"),
+                        ipv, port));
       return true;
     } catch(RecoverableException& ex) {
-      A2_LOG_ERROR(fmt("Failed to setup IPv%d BitTorrent server socket",
-                       family_ == AF_INET?4:6));
-      A2_LOG_ERROR_EX(fmt(MSG_BIND_FAILURE,
-                          getCuid(), port),
-                      ex);
+      A2_LOG_ERROR_EX(fmt("IPv%d BitTorrent: failed to bind TCP port %u",
+                          ipv, port), ex);
       socket_->closeConnection();
     }
   }
@@ -112,14 +107,12 @@ bool PeerListenCommand::execute() {
     return true;
   }
   for(int i = 0; i < 3 && socket_->isReadable(0); ++i) {
-    SocketHandle peerSocket;
+    SharedHandle<SocketCore> peerSocket;
     try {
-      peerSocket.reset(socket_->acceptConnection());
+      peerSocket = socket_->acceptConnection();
       std::pair<std::string, uint16_t> peerInfo;
       peerSocket->getPeerInfo(peerInfo);
 
-      peerSocket->setNonBlockingMode();
-
       SharedHandle<Peer> peer(new Peer(peerInfo.first, peerInfo.second, true));
       cuid_t cuid = e_->newCUID();
       Command* command =
@@ -128,7 +121,7 @@ bool PeerListenCommand::execute() {
       A2_LOG_DEBUG(fmt("Accepted the connection from %s:%u.",
                        peer->getIPAddress().c_str(),
                        peer->getPort()));
-      A2_LOG_DEBUG(fmt("Added CUID#%lld to receive BitTorrent/MSE handshake.",
+      A2_LOG_DEBUG(fmt("Added CUID#%" PRId64 " to receive BitTorrent/MSE handshake.",
                        cuid));
     } catch(RecoverableException& ex) {
       A2_LOG_DEBUG_EX(fmt(MSG_ACCEPT_FAILURE,
