@@ -54,7 +54,6 @@
 #include "DownloadContext.h"
 #include "ServerStatMan.h"
 #include "ServerStat.h"
-#include "PeerStat.h"
 #include "SegmentMan.h"
 #include "FeedbackURISelector.h"
 #include "InorderURISelector.h"
@@ -78,38 +77,55 @@
 #include "UriListParser.h"
 #include "SingletonHolder.h"
 #include "Notifier.h"
+#include "PeerStat.h"
+#include "WrDiskCache.h"
 #ifdef ENABLE_BITTORRENT
 #  include "bittorrent_helper.h"
 #endif // ENABLE_BITTORRENT
 
 namespace aria2 {
 
+namespace {
+template<typename InputIterator>
+void appendReservedGroup(RequestGroupList& list,
+                         InputIterator first, InputIterator last)
+{
+  for(; first != last; ++first) {
+    list.push_back((*first)->getGID(), *first);
+  }
+}
+} // namespace
+
 RequestGroupMan::RequestGroupMan
 (const std::vector<SharedHandle<RequestGroup> >& requestGroups,
  int maxSimultaneousDownloads,
  const Option* option)
-  : reservedGroups_(requestGroups.begin(), requestGroups.end()),
-    maxSimultaneousDownloads_(maxSimultaneousDownloads),
+  : maxSimultaneousDownloads_(maxSimultaneousDownloads),
     option_(option),
     serverStatMan_(new ServerStatMan()),
     maxOverallDownloadSpeedLimit_
     (option->getAsInt(PREF_MAX_OVERALL_DOWNLOAD_LIMIT)),
     maxOverallUploadSpeedLimit_(option->getAsInt
                                 (PREF_MAX_OVERALL_UPLOAD_LIMIT)),
-    rpc_(option->getAsBool(PREF_ENABLE_RPC)),
+    keepRunning_(option->getAsBool(PREF_ENABLE_RPC)),
     queueCheck_(true),
     removedErrorResult_(0),
     removedLastErrorResult_(error_code::FINISHED),
-    maxDownloadResult_(option->getAsInt(PREF_MAX_DOWNLOAD_RESULT))
+    maxDownloadResult_(option->getAsInt(PREF_MAX_DOWNLOAD_RESULT)),
+    wrDiskCache_(0)
 {
-  addRequestGroupIndex(requestGroups);
+  appendReservedGroup(reservedGroups_,
+                      requestGroups.begin(), requestGroups.end());
 }
 
-RequestGroupMan::~RequestGroupMan() {}
+RequestGroupMan::~RequestGroupMan()
+{
+  delete wrDiskCache_;
+}
 
 bool RequestGroupMan::downloadFinished()
 {
-  if(rpc_) {
+  if(keepRunning_) {
     return false;
   }
   return requestGroups_.empty() && reservedGroups_.empty();
@@ -118,199 +134,89 @@ bool RequestGroupMan::downloadFinished()
 void RequestGroupMan::addRequestGroup
 (const SharedHandle<RequestGroup>& group)
 {
-  requestGroups_.push_back(group);
+  requestGroups_.push_back(group->getGID(), group);
 }
 
 void RequestGroupMan::addReservedGroup
 (const std::vector<SharedHandle<RequestGroup> >& groups)
 {
   requestQueueCheck();
-  addRequestGroupIndex(groups);
-  reservedGroups_.insert(reservedGroups_.end(), groups.begin(), groups.end());
+  appendReservedGroup(reservedGroups_, groups.begin(), groups.end());
 }
 
 void RequestGroupMan::addReservedGroup
 (const SharedHandle<RequestGroup>& group)
 {
   requestQueueCheck();
-  addRequestGroupIndex(group);
-  reservedGroups_.push_back(group);
+  reservedGroups_.push_back(group->getGID(), group);
 }
 
+namespace {
+struct RequestGroupKeyFunc {
+  a2_gid_t operator()(const SharedHandle<RequestGroup>& rg) const
+  {
+    return rg->getGID();
+  }
+};
+} // namespace
+
 void RequestGroupMan::insertReservedGroup
 (size_t pos, const std::vector<SharedHandle<RequestGroup> >& groups)
 {
   requestQueueCheck();
-  addRequestGroupIndex(groups);
-  reservedGroups_.insert
-    (reservedGroups_.begin()+std::min(reservedGroups_.size(), pos),
-     groups.begin(), groups.end());
+  pos = std::min(reservedGroups_.size(), pos);
+  reservedGroups_.insert(pos, RequestGroupKeyFunc(),
+                         groups.begin(), groups.end());
 }
 
 void RequestGroupMan::insertReservedGroup
 (size_t pos, const SharedHandle<RequestGroup>& group)
 {
   requestQueueCheck();
-  addRequestGroupIndex(group);
-  reservedGroups_.insert
-    (reservedGroups_.begin()+std::min(reservedGroups_.size(), pos), group);
+  pos = std::min(reservedGroups_.size(), pos);
+  reservedGroups_.insert(pos, group->getGID(), group);
 }
 
-void RequestGroupMan::addRequestGroupIndex
-(const SharedHandle<RequestGroup>& group)
-{
-  assert(groupIndex_.count(group->getGID()) == 0);
-  groupIndex_[group->getGID()] = group;
-}
-
-void RequestGroupMan::addRequestGroupIndex
-(const std::vector<SharedHandle<RequestGroup> >& groups)
-{
-  for(std::vector<SharedHandle<RequestGroup> >::const_iterator i =
-        groups.begin(); i != groups.end(); ++i) {
-    addRequestGroupIndex(*i);
-  }
-}
-
-namespace {
-void removeRequestGroupIndex
-(std::map<a2_gid_t, SharedHandle<RequestGroup> >& groupIndex,
- const SharedHandle<RequestGroup>& group)
-{
-  assert(groupIndex.count(group->getGID()) == 1);
-  groupIndex.erase(group->getGID());
-}
-} // namespace
-
 size_t RequestGroupMan::countRequestGroup() const
 {
   return requestGroups_.size();
 }
 
-SharedHandle<RequestGroup> RequestGroupMan::getRequestGroup(size_t index) const
-{
-  if(index < requestGroups_.size()) {
-    return requestGroups_[index];
-  } else {
-    return SharedHandle<RequestGroup>();
-  }
-}
-
-namespace {
-template<typename Iterator>
-Iterator findByGID(Iterator first, Iterator last, a2_gid_t gid)
-{
-  for(; first != last; ++first) {
-    if((*first)->getGID() == gid) {
-      return first;
-    }
-  }
-  return first;
-}
-} // namespace
-
-SharedHandle<RequestGroup>
-RequestGroupMan::findRequestGroup(a2_gid_t gid) const
-{
-  SharedHandle<RequestGroup> res = findGroup(gid);
-  if(res) {
-    if(res->getState() == RequestGroup::STATE_ACTIVE) {
-      return res;
-    } else {
-      return SharedHandle<RequestGroup>();
-    }
-  } else {
-    return res;
-  }
-}
-
-SharedHandle<RequestGroup>
-RequestGroupMan::findReservedGroup(a2_gid_t gid) const
-{
-  SharedHandle<RequestGroup> res = findGroup(gid);
-  if(res) {
-    if(res->getState() == RequestGroup::STATE_WAITING) {
-      return res;
-    } else {
-      return SharedHandle<RequestGroup>();
-    }
-  } else {
-    return res;
-  }
-}
-
 SharedHandle<RequestGroup> RequestGroupMan::findGroup(a2_gid_t gid) const
 {
-  std::map<a2_gid_t, SharedHandle<RequestGroup> >::const_iterator i =
-    groupIndex_.find(gid);
-  if(i != groupIndex_.end()) {
-    return (*i).second;
-  } else {
-    return SharedHandle<RequestGroup>();
+  SharedHandle<RequestGroup> rg = requestGroups_.get(gid);
+  if(!rg) {
+    rg = reservedGroups_.get(gid);
   }
+  return rg;
 }
 
 size_t RequestGroupMan::changeReservedGroupPosition
-(a2_gid_t gid, int pos, HOW how)
+(a2_gid_t gid, int pos, OffsetMode how)
 {
-  std::deque<SharedHandle<RequestGroup> >::iterator i =
-    findByGID(reservedGroups_.begin(), reservedGroups_.end(), gid);
-  if(i == reservedGroups_.end()) {
-    throw DL_ABORT_EX(fmt("GID#%" PRId64 " not found in the waiting queue.", gid));
-  }
-  SharedHandle<RequestGroup> rg = *i;
-  const size_t maxPos = reservedGroups_.size()-1;
-  if(how == POS_SET) {
-    if(pos < 0) {
-      pos = 0;
-    } else if(pos > 0) {
-      pos = std::min(maxPos, (size_t)pos);
-    }
-  } else if(how == POS_CUR) {
-    size_t abspos = std::distance(reservedGroups_.begin(), i);
-    if(pos < 0) {
-      int dist = -std::distance(reservedGroups_.begin(), i);
-      pos = abspos+std::max(pos, dist);
-    } else if(pos > 0) {
-      int dist = std::distance(i, reservedGroups_.end())-1;
-      pos = abspos+std::min(pos, dist);
-    } else {
-      pos = abspos;
-    }
-  } else if(how == POS_END) {
-    if(pos >= 0) {
-      pos = maxPos;
-    } else {
-      pos = maxPos-std::min(maxPos, (size_t)-pos);
-    }
-  }
-  if(std::distance(reservedGroups_.begin(), i) < pos) {
-    std::rotate(i, i+1, reservedGroups_.begin()+pos+1);
+  ssize_t dest = reservedGroups_.move(gid, pos, how);
+  if(dest == -1) {
+    throw DL_ABORT_EX(fmt("GID#%s not found in the waiting queue.",
+                          GroupId::toHex(gid).c_str()));
   } else {
-    std::rotate(reservedGroups_.begin()+pos, i, i+1);
+    return dest;
   }
-  return pos;
 }
 
 bool RequestGroupMan::removeReservedGroup(a2_gid_t gid)
 {
-  std::deque<SharedHandle<RequestGroup> >::iterator i =
-    findByGID(reservedGroups_.begin(), reservedGroups_.end(), gid);
-  if(i == reservedGroups_.end()) {
-    return false;
-  } else {
-    removeRequestGroupIndex(groupIndex_, *i);
-    reservedGroups_.erase(i);
-    return true;
-  }
+  return reservedGroups_.remove(gid);
 }
 
 namespace {
 
 void notifyDownloadEvent
-(const std::string& event, const SharedHandle<RequestGroup>& group)
+(DownloadEvent event, const SharedHandle<RequestGroup>& group)
 {
-  SingletonHolder<Notifier>::instance()->notifyDownloadEvent(event, group);
+  // Check NULL to make unit test easier.
+  if(SingletonHolder<Notifier>::instance()) {
+    SingletonHolder<Notifier>::instance()->notifyDownloadEvent(event, group);
+  }
 }
 
 } // namespace
@@ -333,12 +239,12 @@ void executeStopHook
     util::executeHookByOptName(group, option, PREF_ON_DOWNLOAD_STOP);
   }
   if(result == error_code::FINISHED) {
-    notifyDownloadEvent(Notifier::ON_DOWNLOAD_COMPLETE, group);
+    notifyDownloadEvent(EVENT_ON_DOWNLOAD_COMPLETE, group);
   } else if(result != error_code::IN_PROGRESS &&
             result != error_code::REMOVED) {
-    notifyDownloadEvent(Notifier::ON_DOWNLOAD_ERROR, group);
+    notifyDownloadEvent(EVENT_ON_DOWNLOAD_ERROR, group);
   } else {
-    notifyDownloadEvent(Notifier::ON_DOWNLOAD_STOP, group);
+    notifyDownloadEvent(EVENT_ON_DOWNLOAD_STOP, group);
   }
 }
 
@@ -348,9 +254,7 @@ namespace {
 class ProcessStoppedRequestGroup {
 private:
   DownloadEngine* e_;
-  std::deque<SharedHandle<DownloadResult> >& downloadResults_;
-  std::deque<SharedHandle<RequestGroup> >& reservedGroups_;
-  std::map<a2_gid_t, SharedHandle<RequestGroup> >& groupIndex_;
+  RequestGroupList& reservedGroups_;
 
   void saveSignature(const SharedHandle<RequestGroup>& group)
   {
@@ -367,21 +271,50 @@ private:
       }
     }
   }
+
+  // Collect statistics during download in PeerStats and update/register
+  // ServerStatMan
+  void collectStat(const SharedHandle<RequestGroup>& group)
+  {
+    if(group->getSegmentMan()) {
+      bool singleConnection =
+        group->getSegmentMan()->getPeerStats().size() == 1;
+      const std::vector<SharedHandle<PeerStat> >& peerStats =
+        group->getSegmentMan()->getFastestPeerStats();
+      for(std::vector<SharedHandle<PeerStat> >::const_iterator i =
+            peerStats.begin(), eoi = peerStats.end(); i != eoi; ++i) {
+        if((*i)->getHostname().empty() || (*i)->getProtocol().empty()) {
+          continue;
+        }
+        int speed = (*i)->getAvgDownloadSpeed();
+        if (speed == 0) continue;
+
+        SharedHandle<ServerStat> ss =
+          e_->getRequestGroupMan()->getOrCreateServerStat((*i)->getHostname(),
+                                                          (*i)->getProtocol());
+        ss->increaseCounter();
+        ss->updateDownloadSpeed(speed);
+        if(singleConnection) {
+          ss->updateSingleConnectionAvgSpeed(speed);
+        }
+        else {
+          ss->updateMultiConnectionAvgSpeed(speed);
+        }
+      }
+    }
+  }
 public:
   ProcessStoppedRequestGroup
   (DownloadEngine* e,
-   std::deque<SharedHandle<DownloadResult> >& downloadResults,
-   std::deque<SharedHandle<RequestGroup> >& reservedGroups,
-   std::map<a2_gid_t, SharedHandle<RequestGroup> >& groupIndex)
+   RequestGroupList& reservedGroups)
     : e_(e),
-      downloadResults_(downloadResults),
-      reservedGroups_(reservedGroups),
-      groupIndex_(groupIndex)
+      reservedGroups_(reservedGroups)
   {}
 
-  void operator()(const SharedHandle<RequestGroup>& group)
+  bool operator()(const RequestGroupList::value_type& group)
   {
     if(group->getNumCommand() == 0) {
+      collectStat(group);
       const SharedHandle<DownloadContext>& dctx = group->getDownloadContext();
       // DownloadContext::resetDownloadStopTime() is only called when
       // download completed. If
@@ -394,9 +327,13 @@ public:
       }
       try {
         group->closeFile();
-        if(group->downloadFinished() &&
+        if(group->isPauseRequested()) {
+          A2_LOG_NOTICE
+            (fmt(_("Download GID#%s paused"),
+                 GroupId::toHex(group->getGID()).c_str()));
+          group->saveControlFile();
+        } else if(group->downloadFinished() &&
            !group->getDownloadContext()->isChecksumVerificationNeeded()) {
-          group->setPauseRequested(false);
           group->applyLastModifiedTimeToLocalFiles();
           group->reportDownloadFinished();
           if(group->allDownloadFinished()) {
@@ -420,8 +357,9 @@ public:
           // we don't remove it.
           if(group->getOption()->getAsBool(PREF_BT_REMOVE_UNSELECTED_FILE) &&
              !group->inMemoryDownload() &&
-             dctx->hasAttribute(bittorrent::BITTORRENT)) {
-            A2_LOG_INFO(fmt(MSG_REMOVING_UNSELECTED_FILE, group->getGID()));
+             dctx->hasAttribute(CTX_ATTR_BT)) {
+            A2_LOG_INFO(fmt(MSG_REMOVING_UNSELECTED_FILE,
+                            GroupId::toHex(group->getGID()).c_str()));
             const std::vector<SharedHandle<FileEntry> >& files =
               dctx->getFileEntries();
             for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
@@ -439,8 +377,8 @@ public:
 #endif // ENABLE_BITTORRENT
         } else {
           A2_LOG_NOTICE
-            (fmt(_("Download GID#%" PRId64 " not complete: %s"),
-                 group->getGID(),
+            (fmt(_("Download GID#%s not complete: %s"),
+                 GroupId::toHex(group->getGID()).c_str(),
                  group->getDownloadContext()->getBasePath().c_str()));
           group->saveControlFile();
         }
@@ -449,12 +387,12 @@ public:
       }
       if(group->isPauseRequested()) {
         group->setState(RequestGroup::STATE_WAITING);
-        reservedGroups_.push_front(group);
+        reservedGroups_.push_front(group->getGID(), group);
         group->releaseRuntimeResource(e_);
         group->setForceHaltRequested(false);
         util::executeHookByOptName(group, e_->getOption(),
                                    PREF_ON_DOWNLOAD_PAUSE);
-        notifyDownloadEvent(Notifier::ON_DOWNLOAD_PAUSE, group);
+        notifyDownloadEvent(EVENT_ON_DOWNLOAD_PAUSE, group);
         // TODO Should we have to prepend spend uris to remaining uris
         // in case PREF_REUSE_URI is disabed?
       } else {
@@ -462,89 +400,19 @@ public:
         e_->getRequestGroupMan()->addDownloadResult(dr);
         executeStopHook(group, e_->getOption(), dr->result);
         group->releaseRuntimeResource(e_);
-        removeRequestGroupIndex(groupIndex_, group);
       }
+      return true;
+    } else {
+      return false;
     }
   }
 };
 } // namespace
 
-namespace {
-class CollectServerStat {
-private:
-  RequestGroupMan* requestGroupMan_;
-public:
-  CollectServerStat(RequestGroupMan* requestGroupMan):
-    requestGroupMan_(requestGroupMan) {}
-
-  void operator()(const SharedHandle<RequestGroup>& group)
-  {
-    if(group->getNumCommand() == 0) {
-      // Collect statistics during download in PeerStats and update/register
-      // ServerStatMan
-      if(group->getSegmentMan()) {
-        bool singleConnection =
-          group->getSegmentMan()->getPeerStats().size() == 1;
-        const std::vector<SharedHandle<PeerStat> >& peerStats =
-          group->getSegmentMan()->getFastestPeerStats();
-        for(std::vector<SharedHandle<PeerStat> >::const_iterator i =
-              peerStats.begin(), eoi = peerStats.end(); i != eoi; ++i) {
-          if((*i)->getHostname().empty() || (*i)->getProtocol().empty()) {
-            continue;
-          }
-          int speed = (*i)->getAvgDownloadSpeed();
-          if (speed == 0) continue;
-
-          SharedHandle<ServerStat> ss =
-            requestGroupMan_->getOrCreateServerStat((*i)->getHostname(),
-                                                    (*i)->getProtocol());
-          ss->increaseCounter();
-          ss->updateDownloadSpeed(speed);
-          if(singleConnection) {
-            ss->updateSingleConnectionAvgSpeed(speed);
-          }
-          else {
-            ss->updateMultiConnectionAvgSpeed(speed);
-          }
-        }
-      }
-    }
-  }
-};
-} // namespace
-
-namespace {
-class FindStoppedRequestGroup {
-public:
-  bool operator()(const SharedHandle<RequestGroup>& group) {
-    return group->getNumCommand() == 0;
-  }
-};
-} // namespace
-
-void RequestGroupMan::updateServerStat()
-{
-  std::for_each(requestGroups_.begin(), requestGroups_.end(),
-                CollectServerStat(this));
-}
-
 void RequestGroupMan::removeStoppedGroup(DownloadEngine* e)
 {
   size_t numPrev = requestGroups_.size();
-
-  updateServerStat();
-
-  std::for_each(requestGroups_.begin(), requestGroups_.end(),
-                ProcessStoppedRequestGroup
-                (e, downloadResults_, reservedGroups_, groupIndex_));
-  std::deque<SharedHandle<RequestGroup> >::iterator i =
-    std::remove_if(requestGroups_.begin(),
-                   requestGroups_.end(),
-                   FindStoppedRequestGroup());
-  if(i != requestGroups_.end()) {
-    requestGroups_.erase(i, requestGroups_.end());
-  }
-
+  requestGroups_.remove_if(ProcessStoppedRequestGroup(e, reservedGroups_));
   size_t numRemoved = numPrev-requestGroups_.size();
   if(numRemoved > 0) {
     A2_LOG_DEBUG(fmt("%lu RequestGroup(s) deleted.",
@@ -585,18 +453,18 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
   if(static_cast<size_t>(maxSimultaneousDownloads_) <= requestGroups_.size()) {
     return;
   }
-  std::vector<SharedHandle<RequestGroup> > temp;
   int count = 0;
   int num = maxSimultaneousDownloads_-requestGroups_.size();
+  std::vector<SharedHandle<RequestGroup> > pending;
+
   while(count < num && (uriListParser_ || !reservedGroups_.empty())) {
     if(uriListParser_ && reservedGroups_.empty()) {
       std::vector<SharedHandle<RequestGroup> > groups;
+      // May throw exception
       bool ok = createRequestGroupFromUriListParser(groups, option_,
                                                     uriListParser_.get());
       if(ok) {
-        addRequestGroupIndex(groups);
-        reservedGroups_.insert(reservedGroups_.end(), groups.begin(),
-                               groups.end());
+        appendReservedGroup(reservedGroups_, groups.begin(), groups.end());
       } else {
         uriListParser_.reset();
         if(reservedGroups_.empty()) {
@@ -604,29 +472,22 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
         }
       }
     }
-    SharedHandle<RequestGroup> groupToAdd = reservedGroups_.front();
+    SharedHandle<RequestGroup> groupToAdd = *reservedGroups_.begin();
     reservedGroups_.pop_front();
+    if((keepRunning_ && groupToAdd->isPauseRequested()) ||
+       !groupToAdd->isDependencyResolved()) {
+      pending.push_back(groupToAdd);
+      continue;
+    }
+    // Drop pieceStorage here because paused download holds its
+    // reference.
+    groupToAdd->dropPieceStorage();
+    configureRequestGroup(groupToAdd);
+    groupToAdd->setRequestGroupMan(this);
     std::vector<Command*> commands;
     try {
-      if((rpc_ && groupToAdd->isPauseRequested()) ||
-         !groupToAdd->isDependencyResolved()) {
-        temp.push_back(groupToAdd);
-        continue;
-      }
-      // Drop pieceStorage here because paused download holds its
-      // reference.
-      groupToAdd->dropPieceStorage();
-      configureRequestGroup(groupToAdd);
       createInitialCommand(groupToAdd, commands, e);
-      groupToAdd->setRequestGroupMan(this);
-      if(commands.empty()) {
-        requestQueueCheck();
-      }
-      groupToAdd->setState(RequestGroup::STATE_ACTIVE);
-      requestGroups_.push_back(groupToAdd);
       ++count;
-      e->addCommand(commands);
-      commands.clear();
     } catch(RecoverableException& ex) {
       A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, ex);
       A2_LOG_DEBUG("Deleting temporal commands.");
@@ -634,18 +495,24 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
       commands.clear();
       A2_LOG_DEBUG("Commands deleted");
       groupToAdd->setLastErrorCode(ex.getErrorCode());
-      // We add groupToAdd to e in order to it is processed in
+      // We add groupToAdd to e later in order to it is processed in
       // removeStoppedGroup().
-      groupToAdd->setState(RequestGroup::STATE_ACTIVE);
-      requestGroups_.push_back(groupToAdd);
-      requestQueueCheck();
     }
+    if(commands.empty()) {
+      requestQueueCheck();
+    } else {
+      e->addCommand(commands);
+    }
+    groupToAdd->setState(RequestGroup::STATE_ACTIVE);
+    requestGroups_.push_back(groupToAdd->getGID(), groupToAdd);
+
     util::executeHookByOptName(groupToAdd, e->getOption(),
                                PREF_ON_DOWNLOAD_START);
-    notifyDownloadEvent(Notifier::ON_DOWNLOAD_START, groupToAdd);
+    notifyDownloadEvent(EVENT_ON_DOWNLOAD_START, groupToAdd);
   }
-  if(!temp.empty()) {
-    reservedGroups_.insert(reservedGroups_.begin(), temp.begin(), temp.end());
+  if(!pending.empty()) {
+    reservedGroups_.insert(reservedGroups_.begin(), RequestGroupKeyFunc(),
+                           pending.begin(), pending.end());
   }
   if(count > 0) {
     e->setNoWait(true);
@@ -656,14 +523,15 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
 
 void RequestGroupMan::save()
 {
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
-        requestGroups_.begin(), eoi = requestGroups_.end(); itr != eoi; ++itr) {
-    if((*itr)->allDownloadFinished() &&
-       !(*itr)->getDownloadContext()->isChecksumVerificationNeeded()) {
-      (*itr)->removeControlFile();
+  for(RequestGroupList::iterator itr = requestGroups_.begin(),
+        eoi = requestGroups_.end(); itr != eoi; ++itr) {
+    const SharedHandle<RequestGroup>& rg = *itr;
+    if(rg->allDownloadFinished() &&
+       !rg->getDownloadContext()->isChecksumVerificationNeeded()) {
+      rg->removeControlFile();
     } else {
       try {
-        (*itr)->saveControlFile();
+        rg->saveControlFile();
       } catch(RecoverableException& e) {
         A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
       }
@@ -673,8 +541,8 @@ void RequestGroupMan::save()
 
 void RequestGroupMan::closeFile()
 {
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
-        requestGroups_.begin(), eoi = requestGroups_.end(); itr != eoi; ++itr) {
+  for(RequestGroupList::iterator itr = requestGroups_.begin(),
+        eoi = requestGroups_.end(); itr != eoi; ++itr) {
     (*itr)->closeFile();
   }
 }
@@ -686,84 +554,124 @@ RequestGroupMan::DownloadStat RequestGroupMan::getDownloadStat() const
   int inprogress = 0;
   int removed = 0;
   error_code::Value lastError = removedLastErrorResult_;
-  for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
-        downloadResults_.begin(), eoi = downloadResults_.end();
-      itr != eoi; ++itr) {
-    if((*itr)->belongsTo != 0) {
+  for(DownloadResultList::const_iterator itr =
+        downloadResults_.begin(), eoi = downloadResults_.end(); itr != eoi;
+      ++itr) {
+    const SharedHandle<DownloadResult>& dr = *itr;
+    if(dr->belongsTo != 0) {
       continue;
     }
-    if((*itr)->result == error_code::FINISHED) {
+    if(dr->result == error_code::FINISHED) {
       ++finished;
-    } else if((*itr)->result == error_code::IN_PROGRESS) {
+    } else if(dr->result == error_code::IN_PROGRESS) {
       ++inprogress;
-    } else if((*itr)->result == error_code::REMOVED) {
+    } else if(dr->result == error_code::REMOVED) {
       ++removed;
     } else {
       ++error;
-      lastError = (*itr)->result;
+      lastError = dr->result;
     }
   }
-  return DownloadStat(finished, error, inprogress, removed,
+  return DownloadStat(error, inprogress,
                       reservedGroups_.size(),
                       lastError);
 }
 
+enum DownloadResultStatus {
+  A2_STATUS_OK,
+  A2_STATUS_INPR,
+  A2_STATUS_RM,
+  A2_STATUS_ERR
+};
+
+namespace {
+const char* getStatusStr(DownloadResultStatus status, bool useColor)
+{
+  // status string is formatted in 4 characters wide.
+  switch(status) {
+  case(A2_STATUS_OK):
+    if(useColor) {
+      return "\033[1;32mOK\033[0m  ";
+    } else {
+      return "OK  ";
+    }
+  case(A2_STATUS_INPR):
+    if(useColor) {
+      return "\033[1;34mINPR\033[0m";
+    } else {
+      return "INPR";
+    }
+  case(A2_STATUS_RM):
+    if(useColor) {
+      return "\033[1mRM\033[0m  ";
+    } else {
+      return "RM  ";
+    }
+  case(A2_STATUS_ERR):
+    if(useColor) {
+      return "\033[1;31mERR\033[0m ";
+    } else {
+      return "ERR ";
+    }
+  default:
+    return "";
+  }
+}
+} // namespace
+
 void RequestGroupMan::showDownloadResults(OutputFile& o, bool full) const
 {
-  static const std::string MARK_OK("OK");
-  static const std::string MARK_ERR("ERR");
-  static const std::string MARK_INPR("INPR");
-  static const std::string MARK_RM("RM");
-
-#ifdef __MINGW32__
-  int pathRowSize = 58;
-#else // !__MINGW32__
-  int pathRowSize = 59;
-#endif // !__MINGW32__
+  int pathRowSize = 55;
   // Download Results:
   // idx|stat|path/length
   // ===+====+=======================================================================
   o.printf("\n%s"
-           "\ngid|stat|avg speed  |",
+           "\ngid   |stat|avg speed  |",
            _("Download Results:"));
   if(full) {
     o.write("  %|path/URI"
-            "\n===+====+===========+===+");
+            "\n======+====+===========+===+");
     pathRowSize -= 4;
   } else {
     o.write("path/URI"
-            "\n===+====+===========+");
+            "\n======+====+===========+");
   }
   std::string line(pathRowSize, '=');
   o.printf("%s\n", line.c_str());
+  bool useColor = o.supportsColor();
   int ok = 0;
   int err = 0;
   int inpr = 0;
   int rm = 0;
-  for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
-        downloadResults_.begin(), eoi = downloadResults_.end();
-      itr != eoi; ++itr) {
-    if((*itr)->belongsTo != 0) {
+  for(DownloadResultList::const_iterator itr =
+        downloadResults_.begin(), eoi = downloadResults_.end(); itr != eoi;
+      ++itr) {
+    const SharedHandle<DownloadResult>& dr = *itr;
+    if(dr->belongsTo != 0) {
       continue;
     }
-    std::string status;
-    if((*itr)->result == error_code::FINISHED) {
-      status = MARK_OK;
+    const char* status;
+    switch(dr->result) {
+    case error_code::FINISHED:
+      status = getStatusStr(A2_STATUS_OK, useColor);
       ++ok;
-    } else if((*itr)->result == error_code::IN_PROGRESS) {
-      status = MARK_INPR;
+      break;
+    case error_code::IN_PROGRESS:
+      status = getStatusStr(A2_STATUS_INPR, useColor);
       ++inpr;
-    } else if((*itr)->result == error_code::REMOVED) {
-      status = MARK_RM;
+      break;
+    case error_code::REMOVED:
+      status = getStatusStr(A2_STATUS_RM, useColor);
       ++rm;
-    } else {
-      status = MARK_ERR;
+      break;
+    default:
+      status = getStatusStr(A2_STATUS_ERR, useColor);
       ++err;
     }
     if(full) {
-      formatDownloadResultFull(o, status, *itr);
+      formatDownloadResultFull(o, status, dr);
     } else {
-      o.write(formatDownloadResult(status, *itr).c_str());
+      o.write(formatDownloadResult(status, dr).c_str());
       o.write("\n");
     }
   }
@@ -788,10 +696,10 @@ void RequestGroupMan::showDownloadResults(OutputFile& o, bool full) const
 namespace {
 void formatDownloadResultCommon
 (std::ostream& o,
- const std::string& status,
- const DownloadResultHandle& downloadResult)
+ const char* status,
+ const SharedHandle<DownloadResult>& downloadResult)
 {
-  o << std::setw(3) << downloadResult->gid << "|"
+  o << std::setw(3) << downloadResult->gid->toAbbrevHex() << "|"
     << std::setw(4) << status << "|"
     << std::setw(11);
   if(downloadResult->sessionTime > 0) {
@@ -807,8 +715,8 @@ void formatDownloadResultCommon
 
 void RequestGroupMan::formatDownloadResultFull
 (OutputFile& out,
- const std::string& status,
- const DownloadResultHandle& downloadResult) const
+ const char* status,
+ const SharedHandle<DownloadResult>& downloadResult) const
 {
   BitfieldMan bt(downloadResult->pieceLength, downloadResult->totalLength);
   bt.setBitfield(reinterpret_cast<const unsigned char*>
@@ -849,8 +757,8 @@ void RequestGroupMan::formatDownloadResultFull
 }
 
 std::string RequestGroupMan::formatDownloadResult
-(const std::string& status,
- const DownloadResultHandle& downloadResult) const
+(const char* status,
+ const SharedHandle<DownloadResult>& downloadResult) const
 {
   std::stringstream o;
   formatDownloadResultCommon(o, status, downloadResult);
@@ -885,11 +793,12 @@ bool RequestGroupMan::isSameFileBeingDownloaded(RequestGroup* requestGroup) cons
     return false;
   }
   std::vector<std::string> files;
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
-        requestGroups_.begin(), eoi = requestGroups_.end(); itr != eoi; ++itr) {
-    if((*itr).get() != requestGroup) {
+  for(RequestGroupList::const_iterator itr = requestGroups_.begin(),
+        eoi = requestGroups_.end(); itr != eoi; ++itr) {
+    const SharedHandle<RequestGroup>& rg = *itr;
+    if(rg.get() != requestGroup) {
       const std::vector<SharedHandle<FileEntry> >& entries =
-        (*itr)->getDownloadContext()->getFileEntries();
+        rg->getDownloadContext()->getFileEntries();
       std::transform(entries.begin(), entries.end(),
                      std::back_inserter(files),
                      mem_fun_sh(&FileEntry::getPath));
@@ -904,87 +813,51 @@ bool RequestGroupMan::isSameFileBeingDownloaded(RequestGroup* requestGroup) cons
 
 void RequestGroupMan::halt()
 {
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
-        requestGroups_.begin(), eoi = requestGroups_.end(); i != eoi; ++i) {
+  for(RequestGroupList::const_iterator i = requestGroups_.begin(),
+        eoi = requestGroups_.end(); i != eoi; ++i) {
     (*i)->setHaltRequested(true);
   }
 }
 
 void RequestGroupMan::forceHalt()
 {
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
-        requestGroups_.begin(), eoi = requestGroups_.end(); i != eoi; ++i) {
+  for(RequestGroupList::const_iterator i = requestGroups_.begin(),
+        eoi = requestGroups_.end(); i != eoi; ++i) {
     (*i)->setForceHaltRequested(true);
   }
 }
 
 TransferStat RequestGroupMan::calculateStat()
 {
-  TransferStat s;
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
-        requestGroups_.begin(), eoi = requestGroups_.end(); i != eoi; ++i) {
-    s += (*i)->calculateStat();
-  }
-  return s;
+  // TODO Currently, all time upload length is not set.
+  return netStat_.toTransferStat();
 }
 
 SharedHandle<DownloadResult>
 RequestGroupMan::findDownloadResult(a2_gid_t gid) const
 {
-  for(std::deque<SharedHandle<DownloadResult> >::const_iterator i =
-        downloadResults_.begin(), eoi = downloadResults_.end(); i != eoi; ++i) {
-    if((*i)->gid == gid) {
-      return *i;
-    }
-  }
-  return SharedHandle<DownloadResult>();
+  return downloadResults_.get(gid);
 }
 
 bool RequestGroupMan::removeDownloadResult(a2_gid_t gid)
 {
-  for(std::deque<SharedHandle<DownloadResult> >::iterator i =
-        downloadResults_.begin(), eoi = downloadResults_.end(); i != eoi; ++i) {
-    if((*i)->gid == gid) {
-      downloadResults_.erase(i);
-      return true;
-    }
-  }
-  return false;
+  return downloadResults_.remove(gid);
 }
 
 void RequestGroupMan::addDownloadResult(const SharedHandle<DownloadResult>& dr)
 {
-  if(maxDownloadResult_ == 0) {
-    if(!downloadResults_.empty()) {
-      for(std::deque<SharedHandle<DownloadResult> >::iterator i =
-            downloadResults_.begin(), eoi = downloadResults_.end(); i != eoi;
-          ++i) {
-        if((*i)->belongsTo == 0 && (*i)->result != error_code::FINISHED) {
-          removedLastErrorResult_ = (*i)->result;
-          ++removedErrorResult_;
-        }
-      }
-      downloadResults_.clear();
-    }
+  bool rv = downloadResults_.push_back(dr->gid->getNumericId(), dr);
+  assert(rv);
+  while(downloadResults_.size() > maxDownloadResult_){
+    DownloadResultList::iterator i = downloadResults_.begin();
+    // Save last encountered error code so that we can report it
+    // later.
+    const SharedHandle<DownloadResult>& dr = *i;
     if(dr->belongsTo == 0 && dr->result != error_code::FINISHED) {
       removedLastErrorResult_ = dr->result;
       ++removedErrorResult_;
     }
-  } else {
-    int curSize = downloadResults_.size();
-    if(curSize >= maxDownloadResult_) {
-      std::deque<SharedHandle<DownloadResult> >::iterator last =
-        downloadResults_.begin()+curSize-maxDownloadResult_+1;
-      for(std::deque<SharedHandle<DownloadResult> >::iterator i =
-            downloadResults_.begin(); i != last; ++i) {
-        if((*i)->belongsTo == 0 && (*i)->result != error_code::FINISHED) {
-          removedLastErrorResult_ = (*i)->result;
-          ++removedErrorResult_;
-        }
-      }
-      downloadResults_.erase(downloadResults_.begin(), last);
-    }
-    downloadResults_.push_back(dr);
+    downloadResults_.pop_front();
   }
 }
 
@@ -1035,13 +908,13 @@ void RequestGroupMan::removeStaleServerStat(time_t timeout)
 bool RequestGroupMan::doesOverallDownloadSpeedExceed()
 {
   return maxOverallDownloadSpeedLimit_ > 0 &&
-    maxOverallDownloadSpeedLimit_ < calculateStat().getDownloadSpeed();
+    maxOverallDownloadSpeedLimit_ < netStat_.calculateDownloadSpeed();
 }
 
 bool RequestGroupMan::doesOverallUploadSpeedExceed()
 {
   return maxOverallUploadSpeedLimit_ > 0 &&
-    maxOverallUploadSpeedLimit_ < calculateStat().getUploadSpeed();
+    maxOverallUploadSpeedLimit_ < netStat_.calculateUploadSpeed();
 }
 
 void RequestGroupMan::getUsedHosts
@@ -1052,28 +925,33 @@ void RequestGroupMan::getUsedHosts
   // speed. We use -download speed so that we can sort them using
   // operator<().
   std::vector<Triplet<size_t, int, std::string> > tempHosts;
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
-        requestGroups_.begin(), eoi = requestGroups_.end(); i != eoi; ++i) {
+  for(RequestGroupList::const_iterator i = requestGroups_.begin(),
+        eoi = requestGroups_.end(); i != eoi; ++i) {
+    const SharedHandle<RequestGroup>& rg = *i;
     const FileEntry::InFlightRequestSet& inFlightReqs =
-      (*i)->getDownloadContext()->getFirstFileEntry()->getInFlightRequests();
+      rg->getDownloadContext()->getFirstFileEntry()->getInFlightRequests();
     for(FileEntry::InFlightRequestSet::iterator j =
           inFlightReqs.begin(), eoj = inFlightReqs.end(); j != eoj; ++j) {
-      uri::UriStruct us;
-      if(uri::parse(us, (*j)->getUri())) {
+      uri_split_result us;
+      if(uri_split(&us, (*j)->getUri().c_str()) == 0) {
         std::vector<Triplet<size_t, int, std::string> >::iterator k;
         std::vector<Triplet<size_t, int, std::string> >::iterator eok =
           tempHosts.end();
+        std::string host = uri::getFieldString(us, USR_HOST,
+                                               (*j)->getUri().c_str());
         for(k =  tempHosts.begin(); k != eok; ++k) {
-          if((*k).third == us.host) {
+          if((*k).third == host) {
             ++(*k).first;
             break;
           }
         }
         if(k == eok) {
-          SharedHandle<ServerStat> ss = findServerStat(us.host, us.protocol);
+          std::string protocol = uri::getFieldString(us, USR_SCHEME,
+                                                     (*j)->getUri().c_str());
+          SharedHandle<ServerStat> ss = findServerStat(host, protocol);
           int invDlSpeed = (ss && ss->isOK()) ?
             -(static_cast<int>(ss->getDownloadSpeed())) : 0;
-          tempHosts.push_back(makeTriplet(1, invDlSpeed, us.host));
+          tempHosts.push_back(makeTriplet(1, invDlSpeed, host));
         }
       }
     }
@@ -1089,4 +967,13 @@ void RequestGroupMan::setUriListParser
   uriListParser_ = uriListParser;
 }
 
+void RequestGroupMan::initWrDiskCache()
+{
+  assert(wrDiskCache_ == 0);
+  size_t limit = option_->getAsInt(PREF_DISK_CACHE);
+  if(limit > 0) {
+    wrDiskCache_ = new WrDiskCache(limit);
+  }
+}
+
 } // namespace aria2
