@@ -64,9 +64,6 @@ extern const std::string SINGLE;
 
 extern const std::string MULTI;
 
-extern const std::string BITTORRENT;
-
-
 void load(const std::string& torrentFile,
           const SharedHandle<DownloadContext>& ctx,
           const SharedHandle<Option>& option,
@@ -163,6 +160,11 @@ getInfoHash(const SharedHandle<DownloadContext>& downloadContext);
 std::string
 getInfoHashString(const SharedHandle<DownloadContext>& downloadContext);
 
+// Returns 8bytes unsigned integer located at offset pos.  The integer
+// in msg is network byte order. This function converts it into host
+// byte order and returns it.
+uint64_t getLLIntParam(const unsigned char* msg, size_t pos);
+
 // Returns 4bytes unsigned integer located at offset pos.  The integer
 // in msg is network byte order. This function converts it into host
 // byte order and returns it.
@@ -173,6 +175,10 @@ uint32_t getIntParam(const unsigned char* msg, size_t pos);
 // byte order and returns it.
 uint16_t getShortIntParam(const unsigned char* msg, size_t pos);
 
+// Put param at location pointed by dest. param is converted into
+// network byte order.
+void setLLIntParam(unsigned char* dest, uint64_t param);
+
 // Put param at location pointed by dest. param is converted into
 // network byte order.
 void setIntParam(unsigned char* dest, uint32_t param);
@@ -183,7 +189,7 @@ void setShortIntParam(unsigned char* dest, uint16_t param);
 
 // Returns message ID located at first byte:msg[0]
 uint8_t getId(const unsigned char* msg);
-  
+
 void checkIndex(size_t index, size_t pieces);
 void checkBegin(int32_t begin, int32_t pieceLength);
 void checkLength(int32_t length);
@@ -218,15 +224,15 @@ unpackcompact(const unsigned char* compact, int family);
 
 // Throws exception if threshold >= actual
 void assertPayloadLengthGreater
-(size_t threshold, size_t actual, const std::string& msgName);
+(size_t threshold, size_t actual, const char* msgName);
 
 // Throws exception if expected != actual
 void assertPayloadLengthEqual
-(size_t expected, size_t actual, const std::string& msgName);
+(size_t expected, size_t actual, const char* msgName);
 
 // Throws exception if expected is not equal to id from data.
 void assertID
-(uint8_t expected, const unsigned char* data, const std::string& msgName);
+(uint8_t expected, const unsigned char* data, const char* msgName);
 
 // Converts attrs into torrent data. This function does not guarantee
 // the returned string is valid torrent data.
@@ -330,6 +336,9 @@ void extractPeer
 
 int getCompactLength(int family);
 
+// Returns textual representation of the |mode|.
+const char* getModeString(BtFileMode mode);
+
 // Writes the detailed information about torrent loaded in dctx.
 template<typename Output>
 void print(Output& o, const SharedHandle<DownloadContext>& dctx)
@@ -346,7 +355,7 @@ void print(Output& o, const SharedHandle<DownloadContext>& dctx)
   if(!torrentAttrs->createdBy.empty()) {
     o.printf("Created By: %s\n", torrentAttrs->createdBy.c_str());
   }
-  o.printf("Mode: %s\n", torrentAttrs->mode.c_str());
+  o.printf("Mode: %s\n", getModeString(torrentAttrs->mode));
   o.write("Announce:\n");
   for(std::vector<std::vector<std::string> >::const_iterator tierIter =
         torrentAttrs->announceList.begin(),
