@@ -32,72 +32,76 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_MESSAGE_H_
-#define _D_MESSAGE_H_
+#ifndef D_MESSAGE_H
+#define D_MESSAGE_H
 
 #include "common.h"
 
 #define MSG_SEGMENT_DOWNLOAD_COMPLETED                                  \
-  "CUID#%s - The download for one segment completed successfully."
-#define MSG_NO_SEGMENT_AVAILABLE "CUID#%s - No segment available."
-#define MSG_CONNECTING_TO_SERVER "CUID#%s - Connecting to %s:%d"
-#define MSG_REDIRECT "CUID#%s - Redirecting to %s"
-#define MSG_SENDING_REQUEST "CUID#%s - Requesting:\n%s"
-#define MSG_RECEIVE_RESPONSE "CUID#%s - Response received:\n%s"
-#define MSG_DOWNLOAD_ABORTED "CUID#%s - Download aborted. URI=%s"
-#define MSG_RESTARTING_DOWNLOAD "CUID#%s - Restarting the download. URI=%s"
-#define MSG_TORRENT_DOWNLOAD_ABORTED "CUID#%s - Download aborted."
+  "CUID#%" PRId64 " - The download for one segment completed successfully."
+#define MSG_NO_SEGMENT_AVAILABLE "CUID#%" PRId64 " - No segment available."
+#define MSG_CONNECTING_TO_SERVER "CUID#%" PRId64 " - Connecting to %s:%d"
+#define MSG_REDIRECT "CUID#%" PRId64 " - Redirecting to %s"
+#define MSG_SENDING_REQUEST "CUID#%" PRId64 " - Requesting:\n%s"
+#define MSG_RECEIVE_RESPONSE "CUID#%" PRId64 " - Response received:\n%s"
+#define MSG_DOWNLOAD_ABORTED "CUID#%" PRId64 " - Download aborted. URI=%s"
+#define MSG_RESTARTING_DOWNLOAD "CUID#%" PRId64 " - Restarting the download. URI=%s"
+#define MSG_TORRENT_DOWNLOAD_ABORTED "CUID#%" PRId64 " - Download aborted."
 #define MSG_MAX_TRY                                                     \
-  "CUID#%s - %d times attempted, but no success. Download aborted."
-#define MSG_SEND_PEER_MESSAGE "CUID#%s - To: %s:%d %s"
-#define MSG_RECEIVE_PEER_MESSAGE "CUID#%s - From: %s:%d %s"
-#define MSG_GOT_NEW_PIECE "CUID#%s - we got new piece. index=%d"
-#define MSG_GOT_WRONG_PIECE "CUID#%s - we got wrong piece. index=%d"
-#define MSG_DOWNLOAD_NOT_COMPLETE "CUID#%s - Download not complete: %s"
+  "CUID#%" PRId64 " - %d times attempted, but no success. Download aborted."
+#define MSG_SEND_PEER_MESSAGE "CUID#%" PRId64 " - To: %s:%d %s"
+#define MSG_RECEIVE_PEER_MESSAGE "CUID#%" PRId64 " - From: %s:%d %s"
+#define MSG_GOT_NEW_PIECE "CUID#%" PRId64 " - we got new piece. index=%lu"
+#define MSG_GOT_WRONG_PIECE "CUID#%" PRId64 " - we got wrong piece. index=%lu"
+#define MSG_DOWNLOAD_NOT_COMPLETE "CUID#%" PRId64 " - Download not complete: %s"
 #define MSG_DOWNLOAD_ALREADY_COMPLETED _("GID#%s - Download has already completed: %s")
-#define MSG_RESOLVING_HOSTNAME "CUID#%s - Resolving hostname %s"
+#define MSG_RESOLVING_HOSTNAME "CUID#%" PRId64 " - Resolving hostname %s"
 #define MSG_NAME_RESOLUTION_COMPLETE                    \
-  "CUID#%s - Name resolution complete: %s -> %s"
+  "CUID#%" PRId64 " - Name resolution complete: %s -> %s"
 #define MSG_NAME_RESOLUTION_FAILED                      \
-  "CUID#%s - Name resolution for %s failed:%s"
-#define MSG_DNS_CACHE_HIT "CUID#%s - DNS cache hit: %s -> %s"
-#define MSG_CONNECTING_TO_PEER "CUID#%s - Connecting to the peer %s"
+  "CUID#%" PRId64 " - Name resolution for %s failed:%s"
+#define MSG_DNS_CACHE_HIT "CUID#%" PRId64 " - DNS cache hit: %s -> %s"
+#define MSG_CONNECTING_TO_PEER "CUID#%" PRId64 " - Connecting to the peer %s"
 #define MSG_PIECE_RECEIVED                                              \
-  "CUID#%s - Piece received. index=%d, begin=%d, length=%d, offset=%llu," \
-  " blockIndex=%d"
-#define MSG_PIECE_BITFIELD "CUID#%s - Piece bitfield %s"
+  "CUID#%" PRId64 " - Piece received. index=%lu, begin=%d, length=%d, offset=%" PRId64 "," \
+  " blockIndex=%lu"
+#define MSG_PIECE_BITFIELD "CUID#%" PRId64 " - Piece bitfield %s"
 #define MSG_REJECT_PIECE_CHOKED                                         \
-  "CUID#%s - Reject piece message in queue because the peer has been choked." \
-  " index=%d, begin=%d, length=%d"
+  "CUID#%" PRId64 " - Reject piece message in queue because the peer has been" \
+  " choked. index=%lu, begin=%d, length=%d"
 #define MSG_REJECT_PIECE_CANCEL                                         \
-  "CUID#%s - Reject piece message in queue because cancel message received." \
-  " index=%d, begin=%d, length=%d"
+  "CUID#%" PRId64 " - Reject piece message in queue because cancel message received." \
+  " index=%lu, begin=%d, length=%d"
 #define MSG_FILE_VALIDATION_FAILURE                             \
-  "CUID#%s - Exception caught while validating file integrity."
-#define MSG_PEER_INTERESTED "CUID#%s - Interested in the peer"
-#define MSG_PEER_NOT_INTERESTED "CUID#%s - Not interested in the peer"
-#define MSG_DELETING_REQUEST_SLOT "CUID#%s - Deleting request slot index=%d, blockIndex=%d"
-#define MSG_DELETING_REQUEST_SLOT_CHOKED "CUID#%s - Deleting request slot index=%d, blockIndex=%d because localhost got choked."
-#define MSG_DELETING_REQUEST_SLOT_TIMEOUT "CUID#%s - Deleting request slot blockIndex=%d because of time out"
-#define MSG_DELETING_REQUEST_SLOT_ACQUIRED "CUID#%s - Deleting request slot blockIndex=%d because the block has been acquired."
-#define MSG_FAST_EXTENSION_ENABLED "CUID#%s - Fast extension enabled."
-#define MSG_EXTENDED_MESSAGING_ENABLED "CUID#%s - Extended Messaging enabled."
+  "CUID#%" PRId64 " - Exception caught while validating file integrity."
+#define MSG_PEER_INTERESTED "CUID#%" PRId64 " - Interested in the peer"
+#define MSG_PEER_NOT_INTERESTED "CUID#%" PRId64 " - Not interested in the peer"
+#define MSG_DELETING_REQUEST_SLOT "CUID#%" PRId64 " - Deleting request slot" \
+  " index=%lu, begin=%d, blockIndex=%lu"
+#define MSG_DELETING_REQUEST_SLOT_CHOKED "CUID#%" PRId64 " - Deleting request slot" \
+  " index=%lu, begin=%d, blockIndex=%lu because localhost got choked."
+#define MSG_DELETING_REQUEST_SLOT_TIMEOUT "CUID#%" PRId64 " - Deleting request slot" \
+  " index=%lu, begin=%d, blockIndex=%lu because of time out"
+#define MSG_DELETING_REQUEST_SLOT_ACQUIRED "CUID#%" PRId64 " - Deleting request slot" \
+  " index=%lu, begin=%d, blockIndex=%lu because the block has been acquired."
+#define MSG_FAST_EXTENSION_ENABLED "CUID#%" PRId64 " - Fast extension enabled."
+#define MSG_EXTENDED_MESSAGING_ENABLED "CUID#%" PRId64 " - Extended Messaging enabled."
 #define MSG_FILE_ALLOCATION_FAILURE                             \
-  "CUID#%s - Exception caught while allocating file space."
+  "CUID#%" PRId64 " - Exception caught while allocating file space."
 #define MSG_CONTENT_DISPOSITION_DETECTED                        \
-  "CUID#%s - Content-Disposition detected. Use %s as filename"
-#define MSG_PEER_BANNED "CUID#%s - Peer %s:%d banned."
+  "CUID#%" PRId64 " - Content-Disposition detected. Use %s as filename"
+#define MSG_PEER_BANNED "CUID#%" PRId64 " - Peer %s:%d banned."
 #define MSG_LISTENING_PORT                                      \
-  "CUID#%s - Using port %d for accepting new connections"
-#define MSG_BIND_FAILURE "CUID#%s - An error occurred while binding port=%d"
+  "CUID#%" PRId64 " - Using port %d for accepting new connections"
+#define MSG_BIND_FAILURE "CUID#%" PRId64 " - An error occurred while binding port=%d"
 #define MSG_INCOMING_PEER_CONNECTION                            \
-  "CUID#%s - Incoming connection, adding new command CUID#%s"
-#define MSG_ACCEPT_FAILURE "CUID#%s - Error in accepting connection"
+  "CUID#%" PRId64 " - Incoming connection, adding new command CUID#%" PRId64 ""
+#define MSG_ACCEPT_FAILURE "CUID#%" PRId64 " - Error in accepting connection"
 #define MSG_TRACKER_RESPONSE_PROCESSING_FAILED                  \
-  "CUID#%s - Error occurred while processing tracker response."
-#define MSG_DHT_ENABLED_PEER "CUID#%s - The peer is DHT-enabled."
+  "CUID#%" PRId64 " - Error occurred while processing tracker response."
+#define MSG_DHT_ENABLED_PEER "CUID#%" PRId64 " - The peer is DHT-enabled."
 #define MSG_CONNECT_FAILED_AND_RETRY            \
-  "CUID#%s - Could not to connect to %s:%u. Trying another address"
+  "CUID#%" PRId64 " - Could not to connect to %s:%u. Trying another address"
 
 #define MSG_UNRECOGNIZED_URI _("Unrecognized URI or unsupported protocol: %s")
 #define MSG_TRACKER_WARNING_MESSAGE _("Tracker returned warning message: %s")
@@ -119,11 +123,11 @@
 #define MSG_DELETING_USED_PIECE _("Deleting used piece index=%d, fillRate(%%)=%d<=%d")
 #define MSG_SELECTIVE_DOWNLOAD_COMPLETED _("Download of selected files was complete.")
 #define MSG_DOWNLOAD_COMPLETED _("The download was complete.")
-#define MSG_REMOVED_HAVE_ENTRY _("Removed %d have entries.")
+#define MSG_REMOVED_HAVE_ENTRY _("Removed %lu have entries.")
 #define MSG_VALIDATING_FILE _("Validating file %s")
-#define MSG_ALLOCATION_COMPLETED _("%d seconds to allocate %s byte(s)")
+#define MSG_ALLOCATION_COMPLETED "%ld seconds to allocate %" PRId64 " byte(s)"
 #define MSG_FILE_ALLOCATION_DISPATCH                    \
-  "Dispatching FileAllocationCommand for CUID#%s."
+  "Dispatching FileAllocationCommand for CUID#%" PRId64 "."
 #define MSG_METALINK_QUEUEING _("Metalink: Queueing %s for download.")
 #define MSG_FILE_DOWNLOAD_COMPLETED _("Download complete: %s")
 #define MSG_SEEDING_END _("Seeding is over.")
@@ -141,13 +145,13 @@
 #define MSG_RESOURCE_NOT_FOUND _("Resource not found")
 #define MSG_FILE_RENAMED _("File already exists. Renamed to %s.")
 #define MSG_CANNOT_PARSE_METALINK _("Cannot parse metalink XML file. XML may be malformed.")
-#define MSG_TOO_SMALL_PAYLOAD_SIZE _("Too small payload size for %s, size=%d.")
+#define MSG_TOO_SMALL_PAYLOAD_SIZE _("Too small payload size for %s, size=%lu.")
 #define MSG_REMOVED_DEFUNCT_CONTROL_FILE _("Removed the defunct control file %s because the download file %s doesn't exist.")
 #define MSG_SHARE_RATIO_REPORT _("Your share ratio was %.1f, uploaded/downloaded=%sB/%sB")
 #define MSG_MISSING_BT_INFO _("Missing %s in torrent metainfo.")
 #define MSG_NULL_TRACKER_RESPONSE _("Tracker returned null data.")
 #define MSG_WINSOCK_INIT_FAILD _("Windows socket library initialization failed")
-#define MSG_TIME_HAS_PASSED _("%d second(s) has passed. Stopping application.")
+#define MSG_TIME_HAS_PASSED _("%ld second(s) has passed. Stopping application.")
 #define MSG_SIGNATURE_SAVED _("Saved signature as %s. Please note that aria2" \
                               " doesn't verify signatures.")
 #define MSG_SIGNATURE_NOT_SAVED _("Saving signature as %s failed. Maybe file" \
@@ -165,10 +169,13 @@
 #define MSG_ESTABLISHING_CONNECTION_FAILED              \
   _("Failed to establish connection, cause: %s")
 #define MSG_NETWORK_PROBLEM _("Network problem has occurred. cause:%s")
+#define MSG_LOADING_SYSTEM_TRUSTED_CA_CERTS_FAILED                              \
+  _("Failed to load trusted CA certificates from system. Cause: %s")
 #define MSG_LOADING_TRUSTED_CA_CERT_FAILED                              \
   _("Failed to load trusted CA certificates from %s. Cause: %s")
 #define MSG_CERT_VERIFICATION_FAILED                    \
-  _("Certificate verification failed. Cause: %s")
+  _("Certificate verification failed. Cause: %s See --ca-certificate and" \
+    " --check-certificate option.")
 #define MSG_NO_CERT_FOUND _("No certificate found.")
 #define MSG_HOSTNAME_NOT_MATCH _("Hostname not match.")
 #define MSG_NO_FILES_TO_DOWNLOAD _("No files to download.")
@@ -190,6 +197,9 @@
 #define MSG_DIR_TRAVERSAL_DETECTED _("Detected directory traversal directive in %s")
 #define MSG_HASH_CHECK_NOT_DONE                                         \
   "File has already been downloaded but hash check has not been done yet."
+#define MSG_REMOVING_UNSELECTED_FILE _("GID#%s - Removing unselected file.")
+#define MSG_FILE_REMOVED _("File %s removed.")
+#define MSG_FILE_COULD_NOT_REMOVED _("File %s could not be removed.")
 
 #define EX_TIME_OUT _("Timeout.")
 #define EX_INVALID_CHUNK_SIZE _("Invalid chunk size.")
@@ -202,14 +212,14 @@
 #define EX_CONNECTION_FAILED _("Connection failed.")
 #define EX_FILENAME_MISMATCH _("The requested filename and the previously registered one are not same. Expected:%s Actual:%s")
 #define EX_BAD_STATUS _("The response status is not successful. status=%d")
-#define EX_TOO_LARGE_FILE _("Too large file size. size=%s")
+#define EX_TOO_LARGE_FILE "Too large file size. size=%" PRId64 ""
 #define EX_TRANSFER_ENCODING_NOT_SUPPORTED _("Transfer encoding %s is not supported.")
 #define EX_SSL_INIT_FAILURE _("SSL initialization failed: %s")
 #define EX_SSL_IO_ERROR _("SSL I/O error")
 #define EX_SSL_PROTOCOL_ERROR _("SSL protocol error")
 #define EX_SSL_UNKNOWN_ERROR _("SSL unknown error %d")
 #define EX_SSL_CONNECT_ERROR _("SSL initialization failed: OpenSSL connect error %d")
-#define EX_SIZE_MISMATCH _("Size mismatch Expected:%s Actual:%s")
+#define EX_SIZE_MISMATCH "Size mismatch Expected:%" PRId64 " Actual:%" PRId64 ""
 #define EX_AUTH_FAILED _("Authorization failed.")
 #define EX_GOT_EOF _("Got EOF from the server.")
 #define EX_EOF_FROM_PEER _("Got EOF from peer.")
@@ -221,12 +231,11 @@
 #define EX_DATA_READ _("Failed to read data from disk.")
 #define EX_FILE_SHA1SUM _("Failed to calculate SHA1 digest of or a part of the file %s, cause: %s")
 #define EX_FILE_SEEK _("Failed to seek the file %s, cause: %s")
-#define EX_FILE_OFFSET_OUT_OF_RANGE _("The offset is out of range, offset=%s")
+#define EX_FILE_OFFSET_OUT_OF_RANGE "The offset is out of range, offset=%" PRId64 ""
 #define EX_NOT_DIRECTORY _("%s is not a directory.")
 #define EX_MAKE_DIR _("Failed to make the directory %s, cause: %s")
-#define EX_SEGMENT_FILE_OPEN _("Failed to open the segment file %s, cause: %s")
-#define EX_SEGMENT_FILE_WRITE _("Failed to write into the segment file %s, cause: %s")
-#define EX_SEGMENT_FILE_READ _("Failed to read from the segment file %s, cause: %s")
+#define EX_SEGMENT_FILE_WRITE "Failed to write into the segment file %s"
+#define EX_SEGMENT_FILE_READ "Failed to read from the segment file %s"
 
 #define EX_SOCKET_OPEN _("Failed to open a socket, cause: %s")
 #define EX_SOCKET_SET_OPT _("Failed to set a socket option, cause: %s")
@@ -246,23 +255,26 @@
 #define EX_SOCKET_PEEK _("Failed to peek data, cause: %s")
 #define EX_SOCKET_UNKNOWN_ERROR _("Unknown socket error %d (0x%x)")
 #define EX_FILE_ALREADY_EXISTS _("File %s exists, but %s does not exist.")
-#define EX_INVALID_PAYLOAD_SIZE _("Invalid payload size for %s, size=%d. It should be %d.")
+#define EX_INVALID_PAYLOAD_SIZE                                 \
+  _("Invalid payload size for %s, size=%lu. It should be %lu.")
 #define EX_INVALID_BT_MESSAGE_ID _("Invalid ID=%d for %s. It should be %d.")
-#define EX_INVALID_CHUNK_CHECKSUM _("Chunk checksum validation failed. checksumIndex=%d, offset=%s, expectedHash=%s, actualHash=%s")
+#define EX_INVALID_CHUNK_CHECKSUM "Chunk checksum validation failed. checksumIndex=%lu, offset=%" PRId64 ", expectedHash=%s, actualHash=%s"
 #define EX_DOWNLOAD_ABORTED _("Download aborted.")
 #define EX_DUPLICATE_FILE_DOWNLOAD _("File %s is being downloaded by other command.")
 #define EX_INSUFFICIENT_CHECKSUM _("Insufficient checksums.")
 #define EX_TRACKER_FAILURE _("Tracker returned failure reason: %s")
 #define EX_FLOODING_DETECTED _("Flooding detected.")
-#define EX_DROP_INACTIVE_CONNECTION _("Drop connection because no request/piece messages were exchanged in a certain period(%d seconds).")
+#define EX_DROP_INACTIVE_CONNECTION \
+  _("Drop connection because no request/piece messages were exchanged in a" \
+    " certain period(%ld seconds).")
 #define EX_INFOHASH_MISMATCH_IN_SEGFILE _("The infoHash in torrent file doesn't match to one in .aria2 file.")
 #define EX_NO_SUCH_FILE_ENTRY _("No such file entry %s")
 #define EX_TOO_SLOW_DOWNLOAD_SPEED _("Too slow Downloading speed: %d <= %d(B/s), host:%s")
 #define EX_NO_HTTP_REQUEST_ENTRY_FOUND _("No HttpRequestEntry found.")
 #define EX_LOCATION_HEADER_REQUIRED _("Got %d status, but no location header provided.")
-#define EX_INVALID_RANGE_HEADER _("Invalid range header. Request: %s-%s/%s, Response: %s-%s/%s")
+#define EX_INVALID_RANGE_HEADER "Invalid range header. Request: %" PRId64 "-%" PRId64 "/%" PRId64 ", Response: %" PRId64 "-%" PRId64 "/%" PRId64 ""
 #define EX_NO_RESULT_WITH_YOUR_PREFS _("No file matched with your preference.")
 #define EX_EXCEPTION_CAUGHT _("Exception caught")
 #define EX_TOO_LONG_PAYLOAD _("Max payload length exceeded or invalid. length = %u")
 #define EX_FILE_LENGTH_MISMATCH_BETWEEN_LOCAL_AND_REMOTE _("Invalid file length. Cannot continue download %s: local %s, remote %s")
-#endif // _D_MESSAGE_H_
+#endif // D_MESSAGE_H
