@@ -42,11 +42,10 @@
 #else // !__sun
 #ifdef HAVE_SYS_SIGNAL_H
 #  include <sys/signal.h>
-#else // HAVE_SYS_SIGNAL_H
-#  ifdef HAVE_SIGNAL_H
-#    include <signal.h>
-#  endif // HAVE_SIGNAL_H
 #endif // HAVE_SYS_SIGNAL_H
+#ifdef HAVE_SIGNAL_H
+#  include <signal.h>
+#endif // HAVE_SIGNAL_H
 #endif // !__sun
 
 #include <sys/types.h>
